/* linux/arch/arm/mach-s3c2443/mach-yup150.c
 *
 * Copyright (c) 2007 Simtec Electronics
 *	Ben Dooks <ben@simtec.co.uk>
 *
 * http://www.fluff.org/ben/smdk2416/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
*/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/hrtimer.h>
#include <linux/init.h>
#include <linux/serial_core.h>
#include <linux/platform_device.h>
#include <linux/io.h>
#include <linux/fb.h>
#include <linux/i2c.h>
#include <linux/delay.h>
#include <linux/sysdev.h>
#include <linux/pda_power.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/nand_ecc.h>
#include <linux/mtd/partitions.h>
#include <linux/spi/spi.h>
#include <linux/spi/flash.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <mach/hardware.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <plat/regs-serial.h>
#include <mach/regs-gpio.h>
#include <mach/regs-gpiol.h>
#include <mach/regs-fb.h>
#include <mach/regs-irq.h>
#include <mach/regs-s3c2416-clock.h>
#include <mach/leds-gpio.h>

#include <mach/idle.h>
#include <plat/fb.h>
#include <plat/regs-iic.h>
#include <plat/iic.h>

#include <plat/s3c2410.h>
#include <plat/s3c2440.h>
#include <plat/clock.h>
#include <plat/devs.h>
#include <plat/cpu.h>
#include <plat/nand.h>
#include <plat/pm.h>
#include <mach/spi.h>

#ifdef CONFIG_TOUCHSCREEN_S3C2410
#include <mach/ts.h>
#include <../drivers/input/touchscreen/ts_filter_chain.h>
#ifdef CONFIG_TOUCHSCREEN_FILTER
#include <../drivers/input/touchscreen/ts_filter_linear.h>
#include <../drivers/input/touchscreen/ts_filter_mean.h>
#include <../drivers/input/touchscreen/ts_filter_median.h>
#include <../drivers/input/touchscreen/ts_filter_group.h>
#endif /* CONFIG_TOUCHSCREEN_FILTER */
#endif /* CONFIG_TOUCHSCREEN_S3C2410 */

#include <linux/dmard03.h>
#include <linux/i2c/l3g4200d.h>
#include <plat/udc.h>

int yup150_gps_enable_on_suspend = 0;
static int yup150_gps_power_on = 1;

#define SYSTEM_REV_CM31			0
#define SYSTEM_REV_YUP120SN		1
#define SYSTEM_REV_YUP120N		2
#define SYSTEM_REV_YUP160W		3

void smdk2416_hsudc_gpio_init(void)
{
	__raw_writel(__raw_readl(S3C2410_MISCCR)&~(1<<12), S3C2410_MISCCR);
	__raw_writel((__raw_readl(S3C2410_MISCCR)&~0x70)|(1<<6)|(1<<4), S3C2410_MISCCR);
	__raw_writel((__raw_readl(S3C2410_DCLKCON)&~0xff)|(1<<4)|(1<<1)|(1<<0), S3C2410_DCLKCON);
}
void smdk2416_hsudc_gpio_uninit(void)
{
	__raw_writel(__raw_readl(S3C2410_MISCCR)|(1<<12), S3C2410_MISCCR);
}
struct s3c24xx_hsudc_platdata smdk2416_hsudc_platdata = {
	.epnum = 9,
	.gpio_init = smdk2416_hsudc_gpio_init,
	.gpio_uninit = smdk2416_hsudc_gpio_uninit,
};


/*
 * LED configuration
 */
static struct s3c24xx_led_platdata yup110_pdata_led = {
	.gpio	= S3C2443_GPH14,
	.flags	= S3C24XX_LEDF_TRISTATE,
	.name	= "test_led",
};

static struct platform_device yup110_test_led = {
	.name	= "s3c24xx_led",
	.id	= -1,
	.dev	= {
		.platform_data = & yup110_pdata_led,
	},
};


#ifdef CONFIG_TOUCHSCREEN_S3C2410
/*
 * Touchscreen configuration.
 */

#ifdef CONFIG_TOUCHSCREEN_FILTER
const static struct ts_filter_group_configuration yup150_ts_group = {
        .length = 12,
        .close_enough = 10,
        .threshold = 6,         /* At least half of the points in a group. */
        .attempts = 10,
};

const static struct ts_filter_median_configuration yup150_ts_median = {
        .extent = 20,
        .decimation_below = 3,
        .decimation_threshold = 8 * 3,
        .decimation_above = 4,
};

const static struct ts_filter_mean_configuration yup150_ts_mean = {
        .length = 4,
};

const static struct ts_filter_linear_configuration yup150_ts_linear = {
        .constants = {1, 0, 0, 0, 1, 0, 1},     /* Don't modify coords. */
        .coord0 = 0,
        .coord1 = 1,
};
#endif

const static struct ts_filter_chain_configuration yup150_filter_configuration[] =
{
#ifdef CONFIG_TOUCHSCREEN_FILTER
        {&ts_filter_group_api,          &yup150_ts_group.config},
        {&ts_filter_median_api,         &yup150_ts_median.config},
        {&ts_filter_mean_api,           &yup150_ts_mean.config},
        {&ts_filter_linear_api,         &yup150_ts_linear.config},
#endif
        {NULL, NULL},
};

const static struct s3c2410_ts_mach_info yup150_ts_cfg = {
        .delay = 10000,
        .presc = 0xff, /* slow as we can go */
        .filter_config = yup150_filter_configuration,
};
#endif /* CONFIG_TOUCHSCREEN_S3C2410 */


/*
 * UART configuration
 */
static struct s3c2410_uartcfg yup150_uartcfgs[] __initdata = {
	[0] = {	/* UART0 : DEBUG */
		.hwport	     = 0,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	},
	[1] = {	/* UART1 : GPS */
		.hwport	     = 1,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	},
	[2] = {	/* UART2 : RD */
		.hwport	     = 2,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	},
	[3] = {	/* UART3 : console */
		.hwport	     = 3,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	}
};

/*
 * nand configuration
 */
struct mtd_partition yup150_nand_part[] = {
        {
                .name           = "Bootloader",
                .offset         = 0x00000000,
                .size           = 0x00200000,
        },
        {
                .name           = "Front sysimg",
                .offset         = 0x00200000,
                .size           = 0x01800000,
        },
        {
                .name           = "User",
                .offset         = 0x03200000,
                .size           = MTDPART_SIZ_FULL,
        },
        {
                .name           = "Rear sysimg",
                .offset         = 0x01A00000,
                .size           = 0x01800000,
        },
        {
                .name           = "Rear sysimg1", /* for compatibility */
                .offset         = 0x01A00000,
                .size           = 0x01800000,
        },
};

static struct s3c2410_nand_set yup150_nand_sets[] = {
	[0] = {
		.name		= "NAND",
		.nr_chips	= 1,
		.nr_partitions	= ARRAY_SIZE(yup150_nand_part),
		.partitions	= yup150_nand_part,
	},
};

static struct s3c2410_platform_nand yup150_nand_info = {
	.tacls		= 20,
	.twrph0		= 30, /* 50 */
	.twrph1		= 20,
	.nr_sets	= ARRAY_SIZE(yup150_nand_sets),
	.sets		= yup150_nand_sets,
};

/*
 * frame buffer configuration
 */
static void yup150_fb_gpio_setup(void)
{
	int	i;
	for (i = 0; i <= 4; i++)
		s3c2410_gpio_cfgpin(S3C2410_GPIONO(S3C2410_GPIO_BANKC,i),
				    (0x02 << (i*2)));
	for (i = 8; i <= 15; i++)
		s3c2410_gpio_cfgpin(S3C2410_GPIONO(S3C2410_GPIO_BANKC,i),
				    (0x02 << (i*2)));
	for (i = 0; i <= 15; i++)
		s3c2410_gpio_cfgpin(S3C2410_GPIONO(S3C2410_GPIO_BANKD,i),
				    (0x02 << (i*2)));
}

static struct s3c_fb_pd_win yup150_fb_win0 = {
        /* this is to ensure we use win0 */
        .win_mode       = {
                .pixclock       = 116594,
                .left_margin    = 3,
                .right_margin   = 6,
                .upper_margin   = 8,
                .lower_margin   = 5,
                .hsync_len      = 41,
                .vsync_len      = 10,
                .xres           = 240,
                .yres           = 432,
		.refresh	= 65,
        },
	.virtual_x	= 240,
	.virtual_y	= 432 * 2,
        .max_bpp        = 32,
        .default_bpp    = 16,
};

static struct s3c_fb_pd_win yup150N_fb_win0 = {
        /* this is to ensure we use win0 */
        .win_mode       = {
                .pixclock       = 116594,//85869,//76594,
                .left_margin    = 3,
                .right_margin   = 6,
                .upper_margin   = 8,
                .lower_margin   = 5,
                .hsync_len      = 41,
                .vsync_len      = 10,
                .xres           = 240,
                .yres           = 400,
		.refresh	= 65,
        },
	.virtual_x	= 240,
	.virtual_y	= 400 * 2,
        .max_bpp        = 32,
        .default_bpp    = 16,
};
static struct s3c_fb_platdata yup150_lcd_pdata __initdata = {
        .setup_gpio     = yup150_fb_gpio_setup,
        .win[0]         = &yup150_fb_win0,
        .vidcon0        = VIDCON0_VLCKEN,
        .vidcon1        = VIDCON1_INV_VCLK | VIDCON1_INV_HSYNC | VIDCON1_INV_VSYNC,
};
static struct s3c_fb_platdata yup150N_lcd_pdata __initdata = {
        .setup_gpio     = yup150_fb_gpio_setup,
        .win[0]         = &yup150N_fb_win0,
        .vidcon0        = VIDCON0_VLCKEN,
        .vidcon1        = VIDCON1_INV_VCLK | VIDCON1_INV_HSYNC | VIDCON1_INV_VSYNC,
};

/*
 * SPI configuration
 */
static void yup150_spi_gpio_setup(struct s3c2410_spi_info *spi, int enable)
{
	if(system_rev == SYSTEM_REV_YUP160W) {
	    	/* LCD SPI CS */
	    	s3c2410_gpio_cfgpin(S3C2410_GPA3, S3C2410_GPA3_OUT);
		s3c2410_gpio_setpin(S3C2410_GPA3, 1);
		/* Serial Flash SPI CS */
		s3c2410_gpio_cfgpin(S3C2410_GPA4, S3C2410_GPA4_OUT);
		s3c2410_gpio_setpin(S3C2410_GPA4, 1);

	} else {
 	     	/* LCD SPI CS */
	    	s3c2410_gpio_cfgpin(S3C2410_GPE6, S3C2410_GPIO_OUTPUT);
		s3c2410_gpio_setpin(S3C2410_GPE6, 1);
		s3c2410_gpio_pullup(S3C2410_GPE6, S3C24XX_GPIO_PULL_UP);
		/* Serial Flash SPI CS */
		s3c2410_gpio_cfgpin(S3C2410_GPE7, S3C2410_GPIO_OUTPUT);
		s3c2410_gpio_setpin(S3C2410_GPE7, 1);
		s3c2410_gpio_pullup(S3C2410_GPE7, S3C24XX_GPIO_PULL_UP);
	}

        if (enable) {
                s3c2410_gpio_cfgpin(S3C2410_GPE13, S3C2410_GPE13_SPICLK0);
                s3c2410_gpio_cfgpin(S3C2410_GPE12, S3C2410_GPE12_SPIMOSI0);
                s3c2410_gpio_cfgpin(S3C2410_GPE11, S3C2410_GPE11_SPIMISO0);
                s3c2410_gpio_pullup(S3C2410_GPE11, S3C24XX_GPIO_PULL_UP);
                s3c2410_gpio_pullup(S3C2410_GPE13, S3C24XX_GPIO_PULL_UP);

		if(system_rev == SYSTEM_REV_YUP160W) {
		    	/* serial flash hold pin */
		    	s3c2410_gpio_cfgpin(S3C2410_GPA6, S3C2410_GPA6_OUT);
			s3c2410_gpio_setpin(S3C2410_GPA6, 1);
		} else {
		    	/* serial flash hold pin */
		    	s3c2410_gpio_cfgpin(S3C2410_GPE9, S3C2410_GPIO_OUTPUT);
			s3c2410_gpio_setpin(S3C2410_GPE9, 1);
			s3c2410_gpio_pullup(S3C2410_GPE9, S3C24XX_GPIO_PULL_UP);
		}
        } else {
                s3c2410_gpio_cfgpin(S3C2410_GPE13, S3C2410_GPIO_INPUT);
                s3c2410_gpio_cfgpin(S3C2410_GPE11, S3C2410_GPIO_INPUT);
                s3c2410_gpio_pullup(S3C2410_GPE11, S3C24XX_GPIO_PULL_DOWN);
                s3c2410_gpio_pullup(S3C2410_GPE12, S3C24XX_GPIO_PULL_DOWN);
                s3c2410_gpio_pullup(S3C2410_GPE13, S3C24XX_GPIO_PULL_DOWN);
		
		if(system_rev == SYSTEM_REV_YUP160W) {
		    	/* serial flash hold pin */
		    	s3c2410_gpio_cfgpin(S3C2410_GPA6, S3C2410_GPA6_OUT);
			s3c2410_gpio_setpin(S3C2410_GPA6, 0);
		} else {
		    	/* serial flash hold pin */
		    	s3c2410_gpio_cfgpin(S3C2410_GPE9, S3C2410_GPIO_OUTPUT);
			s3c2410_gpio_setpin(S3C2410_GPE9, 0);
			s3c2410_gpio_pullup(S3C2410_GPE9, S3C24XX_GPIO_PULL_UP);
		}
        }
}

static void yup150_spi_set_cs(struct s3c2410_spi_info *spi, int cs, int pol)
{
	if(system_rev == SYSTEM_REV_YUP160W) {
	    	if (cs == 0)
		    	s3c2410_gpio_setpin(S3C2410_GPA3, pol);
		else if (cs == 1)
		    	s3c2410_gpio_setpin(S3C2410_GPA4, pol);
	} else {
	    	if (cs == 0)
		    	s3c2410_gpio_setpin(S3C2410_GPE6, pol);
		else if (cs == 1)
		    	s3c2410_gpio_setpin(S3C2410_GPE7, pol);
	}
 
}

static struct s3c2410_spi_info yup150_spi_pdata = {
	.num_cs		= 2,
	.bus_num	= 0,
	.gpio_setup	= yup150_spi_gpio_setup,
	.set_cs		= yup150_spi_set_cs,
};

/*
 * serial flash configuration
 */
static struct mtd_partition	yup150_spi_flash_parts[] = {
        {
                .name           = "lastpos",
                .offset         = 0x00000,
                .size           = 0x20000,
        },
        {
                .name           = "config",
                .offset         = 0x20000,
                .size           = 0x20000,
        },
};

static struct flash_platform_data       yup150_spi_flash_data = {
        .nr_parts = 2,
        .parts = yup150_spi_flash_parts,
};

static struct spi_board_info yup150_spi_device[] = {
        {
                .modalias = "btl302443",
                .bus_num = 0,
                .chip_select = 0,
                .max_speed_hz = 1 * 1000 * 1000,
        },
        {
                .modalias = "m25p80",
                .bus_num = 0,
                .chip_select = 1,
                .max_speed_hz = 20 * 1000 * 1000,
                .platform_data = &yup150_spi_flash_data,
        },
};

static struct spi_board_info yup110sn_spi_device[] = {
        {
                .modalias = "eh300wq",
                .bus_num = 0,
                .chip_select = 0,
                .max_speed_hz = 1 * 1000 * 1000,
        },
        {
                .modalias = "m25p80",
                .bus_num = 0,
                .chip_select = 1,
                .max_speed_hz = 20 * 1000 * 1000,
                .platform_data = &yup150_spi_flash_data,
        },
};


static struct spi_board_info yup150n_spi_device[] = {
        {
                .modalias = "xt32013a",
                .bus_num = 0,
                .chip_select = 0,
                .max_speed_hz = 1 * 1000 * 1000,
        },
        {
                .modalias = "m25p80",
                .bus_num = 0,
                .chip_select = 1,
                .max_speed_hz = 20 * 1000 * 1000,
                .platform_data = &yup150_spi_flash_data,
        },
};

/*
 * I2C configuration
 */
static struct s3c2410_platform_i2c yup150_i2c_data0 __initdata = {
        .flags          = 0,
        .slave_addr     = 0x10,
        .bus_freq       = 100*1000,
#ifdef CONFIG_TOUCHSCREEN_S3C2410
	/* before PP */
        .max_freq       = 200 * 1000,
#else
	.max_freq	= 500 * 1000,
#endif
        .sda_delay      = S3C2410_IICLC_SDA_DELAY5 | S3C2410_IICLC_FILTER_ON,
};
/*
 * Backlight configuration
 */
static struct platform_device s3c_obd1601muv_device = {
	.name = "obd1601muv",
	.id = -1,
};

static struct platform_device s3c_bd1603_device = { /* YUP150N Backlight */
	.name = "bd1603",
	.id = -1,
};
static struct platform_device s3c_bd1754_device = { /* YUP150N Backlight */
	.name = "bd1754",
	.id = -1,
};
/*
 * YUP110N Button LED driver
 */
static struct platform_device yup110n_led_device = {
	.name = "yup110n_led",
	.id = -1,
};
static struct platform_device yup120x_led_device = {
	.name = "yup120x_led",
	.id = -1,
};
static struct platform_device yup160w_led_device = {
	.name = "yup160w_led",
	.id = -1,
};

/*
 * USB  configuration
 */
static void s3c_udc_pullup(enum s3c2410_udc_cmd_e cmd)
{
	switch (cmd) {
		case S3C2410_UDC_P_ENABLE :
			s3c2410_gpio_setpin(S3C2410_GPG3, 1);
			break;
		case S3C2410_UDC_P_DISABLE :
			s3c2410_gpio_setpin(S3C2410_GPG3, 0);
			break;
		case S3C2410_UDC_P_RESET :
			break;
		default:
			break;
	}
}
static struct platform_device s3c_usbgadget_device = {
	//.name = "s3c2410-usbgadget",
	.name = "s3c-hsudc",
	.id = -1,
};

static int cm31_l3g4200d_init(void)
{
    return 0;
}

static void cm31_l3g4200d_exit(void)
{
}
static int cm31_l3g4200d_power_on(void)
{
    return 0;
}
static int cm31_l3g4200d_power_off(void)
{
    return 0;
}
static struct s3c2410_udc_mach_info s3c_udc_cfg __initdata = {
	.udc_command            = s3c_udc_pullup,
	.vbus_pin               = S3C2410_GPG2,
	.vbus_pin_inverted      = 1,
};

/*
 * wifi power onoff 
 */

static struct platform_device s3c_bp3595_device = {
	.name = "bp3595",
	.id = -1,
};

/*
 * Gyro configuration
 */
static struct platform_device s3c_l3g4200d_device = {
	.name = "l3g4200d_gyr",
	.id = -1,
};

/*
 * accelerometer configuration
 */
static struct platform_device s3c_dmard03_device = {
	.name = "dmard03",
	.id = -1,
};
static struct i2c_board_info ak4183_i2c_dev __initdata = {
	.type = "ak4183",
	.addr = 0x48,
	.irq = IRQ_EINT7,
};

/*
 * remocon configuration
 */
static struct platform_device yup110r_remocon_device = {
	.name = "yup110r-remocon",
	.id = -1,
};

/*
 * button configuration
 */
static struct platform_device yup150_btn_device = {
	.name = "yup150-button",
	.id = -1,
};

/*
 * Power supply configuration
 */
static struct platform_device yup150_power_device = {
	.name			= "yup150-power",
	.id			= -1,
};

static struct platform_device *yup150_devices[] __initdata = {
	&s3c_device_nand,
	&s3c_device_adc,
	&s3c_device_ts,
#ifdef CONFIG_S3C_DEV_HSMMC
	&s3c_device_hsmmc0,
#endif
#ifdef CONFIG_S3C_DEV_HSMMC1
	&s3c_device_hsmmc1,
#endif
	&s3c_device_i2c0,
	&s3c_device_fb,
	&s3c_device_g2d,
	&s3c_device_hsspi,
};

static void __init yup150_map_io(void)
{
	s3c24xx_init_io(NULL, 0);
	s3c24xx_init_clocks(12000000);
	s3c24xx_init_uarts(yup150_uartcfgs, ARRAY_SIZE(yup150_uartcfgs));
}

struct dmard03_platform_data s3c2416_dmard03_platform_data = {
	
	.ctrl_reg1_init         = N_AO4 | TMS_684 | INT1_GND,
	.ctrl_reg2_init         = 0,
	.ctrl_reg3_init         = 0xc8, /* defalut 0x08 */
	.ctrl_reg4_init         = 0x08, /* defalut 0x08 */

};

static struct i2c_board_info i2c_device_dmard03 __initdata = {
	.type = "dmard03",
	.addr = 0x1C,	/* 17pin GND:0x1C/ VCC:0x1D */
	.platform_data = &s3c2416_dmard03_platform_data,
};

static struct l3g4200d_gyr_platform_data cm31_l3g4200d_platform_data ={
    	.init		= cm31_l3g4200d_init,
	.exit		= cm31_l3g4200d_exit,
    	.power_on	= cm31_l3g4200d_power_on,
    	.power_off	= cm31_l3g4200d_power_off,
	.poll_interval	= 10, //2,3,5,10ms
    	.min_interval	= 1, //dummy
    	.fs_range	= L3G4200D_GYR_FS_250DPS,// L3G4200D_GYR_FS_2000DPS,//,L3G4200D_GYR_FS_500DPS
	.axis_map_x     = 0,
      	.axis_map_y     = 1,
	.axis_map_z     = 2,

	.negate_x	= 0,
	.negate_y	= 0,
	.negate_z	= 0,
};

static struct i2c_board_info i2c_device_l3g4200d __initdata = {
	.type = "l3g4200d_gyr",
	.addr = 0x69,	
	.platform_data = &cm31_l3g4200d_platform_data,
};

#ifdef CONFIG_HSMMC_S3C
#include <plat/hsmmc.h>
extern struct s3c_hsmmc_cfg s3c_hsmmc1_platform;
#elif defined(CONFIG_MMC_SDHCI_S3C)
#include <linux/mmc/sdhci.h>
#include <plat/sdhci.h>
#ifdef CONFIG_S3C_DEV_HSMMC1
static struct s3c_sdhci_platdata cm31_hsmmc1_pdata __initdata = {
	.max_width		= 4,
	.cd_type		= S3C_SDHCI_CD_GPIO,
	.ext_cd_gpio		= S3C2410_GPF4,
	.ext_cd_gpio_invert	= 1,
};
#endif
#ifdef CONFIG_S3C_DEV_HSMMC
static struct s3c_sdhci_platdata cm31_hsmmc0_pdata __initdata = {
	.max_width		= 4,
	//.cd_type		= S3C_SDHCI_CD_GPIO,
	.cd_type		= S3C_SDHCI_CD_NONE,
};
#endif
#endif /* CONFIG_HSMMC_S3C */

static void yup150_codec_power_on_off(int on)
{
	s3c2410_gpio_cfgpin(S3C2410_GPB5, S3C2410_GPB5_OUTP);
	if (on) {
		s3c2410_gpio_setpin(S3C2410_GPB5, 1);
		s3c2410_gpio_pullup(S3C2410_GPB5, S3C24XX_GPIO_PULL_UP);
	} else {
		s3c2410_gpio_setpin(S3C2410_GPB5, 0);
		s3c2410_gpio_pullup(S3C2410_GPB5, S3C24XX_GPIO_PULL_DOWN);
	}
}

void yup150_gps_power_on_off(int on)
{
	/* gps enable/disable : GPG7 only */
	s3c2410_gpio_cfgpin(S3C2410_GPG7, S3C2410_GPG7_OUTP);
	if (on) {
		s3c2410_gpio_setpin(S3C2410_GPG7, 1);
		s3c2410_gpio_pullup(S3C2410_GPG7, S3C24XX_GPIO_PULL_UP);
	} else {
		s3c2410_gpio_setpin(S3C2410_GPG7, 0);
		s3c2410_gpio_pullup(S3C2410_GPG7, S3C24XX_GPIO_PULL_DOWN);
	}
}

void yup160w_wifi_reset(void)
{
	/* wifi reset : GPG0 only */
    	s3c2410_gpio_cfgpin(S3C2410_GPG0, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPG0, 0);
	s3c2410_gpio_pullup(S3C2410_GPG0, S3C24XX_GPIO_PULL_DOWN);
	udelay(500);
	s3c2410_gpio_setpin(S3C2410_GPG0, 1);
	s3c2410_gpio_pullup(S3C2410_GPG0, S3C24XX_GPIO_PULL_UP);
}
static void yup150_nand_wp_on_off(int on)
{
	s3c2410_gpio_cfgpin(S3C2410_GPG5, S3C2410_GPG5_OUTP);
	if (on) {
		s3c2410_gpio_setpin(S3C2410_GPG5, 0);
		s3c2410_gpio_pullup(S3C2410_GPG5, S3C24XX_GPIO_PULL_DOWN);
	} else {
		s3c2410_gpio_setpin(S3C2410_GPG5, 1);
		s3c2410_gpio_pullup(S3C2410_GPG5, S3C24XX_GPIO_PULL_UP);
	}
}

static void yup150_sflash_wp_on_off(int on)
{
	if(system_rev == SYSTEM_REV_YUP160W){
	    	s3c2410_gpio_cfgpin(S3C2410_GPA5, S3C2410_GPA5_OUT);
		if (on) {
		    	s3c2410_gpio_setpin(S3C2410_GPA5, 0);
		} else {
		    	s3c2410_gpio_setpin(S3C2410_GPA5, 1);
		}
	} else {
	    	s3c2410_gpio_cfgpin(S3C2410_GPE8, S3C2410_GPE8_OUTP);
		if (on) {
		    	s3c2410_gpio_setpin(S3C2410_GPE8, 0);
			s3c2410_gpio_pullup(S3C2410_GPE8, S3C24XX_GPIO_PULL_DOWN);
		} else {
		    	s3c2410_gpio_setpin(S3C2410_GPE8, 1);
			s3c2410_gpio_pullup(S3C2410_GPE8, S3C24XX_GPIO_PULL_UP);
		}
	}
}

#ifdef CONFIG_PM
static int yup150_pm_suspend(struct sys_device *sd, pm_message_t state)
{
	printk(KERN_INFO "yup150_pm_suspend\n");

#if 0
	/* disable audio codec */
	yup150_codec_power_on_off(0);
#endif

#if 0
	if (! yup150_gps_enable_on_suspend) {
		yup150_gps_power_on_off(0);
	}
#endif

	/* Nand WP enable */
	yup150_nand_wp_on_off(1);

	/* Serial Flash WP enable */
	yup150_sflash_wp_on_off(1);

	return 0;
}

static int yup150_pm_resume(struct sys_device *sd)
{
	printk(KERN_INFO "yup150_pm_resume\n");

	/* enable audio codec */
	if (system_rev != SYSTEM_REV_CM31)
		yup150_codec_power_on_off(1);

	/* gps enable */
	if (yup150_gps_power_on)
		yup150_gps_power_on_off(1);

	/* Nand WP disable */
	yup150_nand_wp_on_off(0);

	/* Serial Flash WP disable */
	yup150_sflash_wp_on_off(0);

	return 0;
}
#else /* CONFIG_PM */
#define yup150_pm_suspend	NULL
#define yup150_pm_resume	NULL
#endif /* CONFIG_PM */

static struct sysdev_class yup150_pm_sysclass = {
	.name		= "mach-yup150",
	.suspend	= yup150_pm_suspend,
	.resume		= yup150_pm_resume,
};

static struct sys_device yup150_pm_sysdev = {
	.cls		= & yup150_pm_sysclass,
};

void yup150_power_led_off(void)
{
	s3c2410_gpio_cfgpin(S3C2410_GPG6, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPG6, 0);
	s3c2410_gpio_pullup(S3C2410_GPG6, S3C24XX_GPIO_PULL_DOWN);
}

static void yup150_power_off(void)
{
	printk(KERN_INFO "powering system down...\n");

	/* gps off */
	yup150_gps_power_on_off(0);

	/* Backlight off */
	s3c2410_gpio_cfgpin(S3C2410_GPB0, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPB0, 0);
	s3c2410_gpio_pullup(S3C2410_GPB0, S3C24XX_GPIO_PULL_DOWN);
	msleep(2);

	/* Power LED off */
	yup150_power_led_off();

	/* SW_OFF */
	s3c2410_gpio_cfgpin(S3C2410_GPA13, S3C2410_GPA13_OUT);
	s3c2410_gpio_setpin(S3C2410_GPA13, 0);

	while (1);
}

static ssize_t gps_enable_on_suspend_show(struct kobject *kobj,
					  struct kobj_attribute *attr,
					  char *buf)
{
	return sprintf(buf, "%d\n", yup150_gps_enable_on_suspend);
}
static ssize_t gps_enable_on_suspend_store(struct kobject *kobj,
					   struct kobj_attribute *attr,
					   const char * buf, size_t n)
{
	int value;
	if (sscanf(buf, "%d", &value) != 1 ||
	    (value != 0 && value != 1))
	{
		printk(KERN_ERR "gps_enable_on_suspend_store: Invalid value\n");
		return -EINVAL;
	}
	yup150_gps_enable_on_suspend = value;
	return n;
}
static ssize_t gps_power_on_show(struct kobject *kobj,
				 struct kobj_attribute *attr,
				 char *buf)
{
	return sprintf(buf, "%d\n", yup150_gps_power_on);
}
static ssize_t gps_power_on_store(struct kobject *kobj,
				  struct kobj_attribute *attr,
				  const char * buf, size_t n)
{
	int value;
	if (sscanf(buf, "%d", &value) != 1 ||
	    (value != 0 && value != 1))
	{
		printk(KERN_ERR "gps_power_on_store: Invalid value\n");
		return -EINVAL;
	}
	yup150_gps_power_on = value;
	yup150_gps_power_on_off(yup150_gps_power_on);
	return n;
}
static struct kobj_attribute gps_enable_on_suspend_attr =
	__ATTR(gps_enable_on_suspend, 0644, gps_enable_on_suspend_show,
					    gps_enable_on_suspend_store);
static struct kobj_attribute gps_power_on_attr =
	__ATTR(gps_power_on, 0644, gps_power_on_show, gps_power_on_store);

static ssize_t rststat_show(struct kobject *kobj, struct kobj_attribute *attr,
			    char *buf)
{
	u32	rststat = __raw_readl(S3C2416_RSTSTAT);

	if (rststat & (1<<0))
		strcpy(buf, "nRESET\n");
	else if (rststat & (1<<2))
		strcpy(buf, "watchdog\n");
	else if (rststat & (1<<3))
		strcpy(buf, "sleep\n");
	else if (rststat & (1<<4))
		strcpy(buf, "deep-stop\n");
	else if (rststat & (1<<5))
		strcpy(buf, "swrst\n");
	else
		strcpy(buf, "unknown\n");

	return strlen(buf);
}
static ssize_t wkupstat_show(struct kobject *kobj, struct kobj_attribute *attr,
			     char *buf)
{
	u32	wkupstat = __raw_readl(S3C2416_WKUPSTAT);

	if (wkupstat & (1<<0))
		strcpy(buf, "eint\n");
	else if (wkupstat & (1<<1))
		strcpy(buf, "rtc\n");
	else if (wkupstat & (1<<4))
		strcpy(buf, "rtc_tick\n");
	else if (wkupstat & (1<<5))
		strcpy(buf, "batf\n");
	else
		strcpy(buf, "unknown\n");

	return strlen(buf);
}
static struct kobj_attribute rststat_attr =
	__ATTR(rststat, 0444, rststat_show, NULL);
static struct kobj_attribute wkupstat_attr =
	__ATTR(wkupstat, 0444, wkupstat_show, NULL);

static void __init yup150_init_pm(void)
{
	sysdev_class_register(&yup150_pm_sysclass);
        sysdev_register(&yup150_pm_sysdev);

	__raw_writel((1<<15), S3C2443_PWRCFG);
	s3c2410_pm_init();

	pm_power_off = yup150_power_off;
}

static void __init yup150_get_system_rev(void)
{
	s3c2410_gpio_pullup(S3C2410_GPB2, S3C24XX_GPIO_PULL_UP);
	s3c2410_gpio_pullup(S3C2410_GPB3, S3C24XX_GPIO_PULL_UP);
	s3c2410_gpio_pullup(S3C2410_GPB4, S3C24XX_GPIO_PULL_UP);

	s3c2410_gpio_cfgpin(S3C2410_GPB2, S3C2410_GPB2_INP);
	s3c2410_gpio_cfgpin(S3C2410_GPB3, S3C2410_GPB3_INP);
	s3c2410_gpio_cfgpin(S3C2410_GPB4, S3C2410_GPB4_INP);

	system_rev = 0;

	if (s3c2410_gpio_getpin(S3C2410_GPB2))
		system_rev |= (1 << 0);
	if (s3c2410_gpio_getpin(S3C2410_GPB3))
		system_rev |= (1 << 1);
	if (s3c2410_gpio_getpin(S3C2410_GPB4))
		system_rev |= (1 << 2);

	s3c2410_gpio_pullup(S3C2410_GPB2, S3C24XX_GPIO_PULL_NONE);
	s3c2410_gpio_pullup(S3C2410_GPB3, S3C24XX_GPIO_PULL_NONE);
	s3c2410_gpio_pullup(S3C2410_GPB4, S3C24XX_GPIO_PULL_NONE);
	printk("system_serial_low: %d\n", system_serial_low);
	printk("system_serial_high : %d\n", system_serial_high);
	printk("System Revision : %d\n", system_rev);
}

static void __init yup150_machine_init(void)
{
	yup150_get_system_rev();

	/* enable audio codec */
	if (system_rev != SYSTEM_REV_CM31)
		yup150_codec_power_on_off(1);

	/* gps enable */
	yup150_gps_power_on_off(1);

	/* Serial Flash WP disable */
	yup150_sflash_wp_on_off(0);
#if 0
	/* for debug */
	s3c2410_gpio_cfgpin(S3C2443_GPH14, S3C2443_GPH14_OUTP);
	s3c2410_gpio_setpin(S3C2443_GPH14, 0);
	s3c2410_gpio_pullup(S3C2443_GPH14, S3C24XX_GPIO_PULL_DOWN);
#endif

	if((system_rev == SYSTEM_REV_YUP120SN) || (system_rev == SYSTEM_REV_YUP120N) 
		|| (system_rev == SYSTEM_REV_YUP160W))
		s3c_fb_set_platdata(&yup150N_lcd_pdata);
	else
		s3c_fb_set_platdata(&yup150_lcd_pdata);


#ifdef CONFIG_TOUCHSCREEN_S3C2410
	set_s3c2410ts_info(&yup150_ts_cfg);
#elif defined(CONFIG_TOUCHSCREEN_AK4183)
	s3c2410_gpio_cfgpin(S3C2410_GPF7, S3C2410_GPF7_EINT7);
	s3c2410_gpio_pullup(S3C2410_GPF7, S3C24XX_GPIO_PULL_UP);
#endif

#ifdef CONFIG_HSMMC_S3C
	s3c2410_gpio_cfgpin(S3C2410_GPF4, S3C2410_GPF4_EINT4);
	s3c2410_gpio_pullup(S3C2410_GPF4, S3C24XX_GPIO_PULL_UP);
	s3c_hsmmc1_platform.irq_cd = IRQ_EINT4;
#elif defined(CONFIG_MMC_SDHCI_S3C)
	s3c2410_gpio_cfgpin(S3C2410_GPG0, S3C2410_GPG0_EINT8);
	s3c2410_gpio_pullup(S3C2410_GPG0, S3C24XX_GPIO_PULL_UP);
	
	s3c2410_gpio_cfgpin(S3C2410_GPF4, S3C2410_GPF4_EINT4);
	s3c2410_gpio_pullup(S3C2410_GPF4, S3C24XX_GPIO_PULL_UP);
#ifdef CONFIG_S3C_DEV_HSMMC
	s3c_sdhci0_set_platdata(&cm31_hsmmc0_pdata);
	/* restriction on hsmmc clock */
 	{
	    	struct s3c_sdhci_platdata *pdata = s3c_device_hsmmc0.dev.platform_data;
		pdata->clocks[0] = "hsmmc";
		pdata->clocks[1] = NULL;
		pdata->clocks[2] = NULL;
		pdata->clocks[3] = NULL;
		if(system_rev == SYSTEM_REV_YUP160W) {
		    	pdata->host_quirks = SDHCI_QUIRK_NO_MULTIBLOCK;
		}
        }
#endif
#ifdef CONFIG_S3C_DEV_HSMMC1
	s3c_sdhci1_set_platdata(&cm31_hsmmc1_pdata);
	/* restriction on hsmmc clock */
 	{
	    	struct s3c_sdhci_platdata *pdata = s3c_device_hsmmc1.dev.platform_data;
		pdata->clocks[0] = "hsmmc";
		pdata->clocks[1] = NULL;
		pdata->clocks[2] = NULL;
		pdata->clocks[3] = NULL;
        }
#endif
#endif /* CONFIG_HSMMC_S3C */
	s3c_device_nand.dev.platform_data = &yup150_nand_info;
	s3c_i2c0_set_platdata(&yup150_i2c_data0);
	s3c_device_hsspi.dev.platform_data = &yup150_spi_pdata;
	platform_add_devices(yup150_devices, ARRAY_SIZE(yup150_devices));
	
	if (system_rev == SYSTEM_REV_YUP120SN){
		spi_register_board_info(yup150n_spi_device, ARRAY_SIZE(yup150n_spi_device));
		platform_device_register(&s3c_bd1603_device);
		platform_device_register(&s3c_dmard03_device),
		platform_device_register(&s3c_l3g4200d_device);
		i2c_register_board_info(0, &ak4183_i2c_dev, 1);
		i2c_register_board_info(0, &i2c_device_dmard03, 1);
		i2c_register_board_info(0, &i2c_device_l3g4200d, 1);
		platform_device_register(&yup110_test_led);
	}else if (system_rev == SYSTEM_REV_YUP120N){
		spi_register_board_info(yup150n_spi_device, ARRAY_SIZE(yup150n_spi_device));
		platform_device_register(&s3c_bd1754_device);
		platform_device_register(&s3c_dmard03_device);
		platform_device_register(&s3c_l3g4200d_device);
		i2c_register_board_info(0, &ak4183_i2c_dev, 1);
		i2c_register_board_info(0, &i2c_device_dmard03, 1);
		i2c_register_board_info(0, &i2c_device_l3g4200d, 1);
		platform_device_register(&yup110n_led_device);
		platform_device_register(&yup120x_led_device);
		platform_device_register(&yup110_test_led);
	} else if (system_rev == SYSTEM_REV_YUP160W){
		spi_register_board_info(yup150n_spi_device, ARRAY_SIZE(yup150n_spi_device));
		platform_device_register(&s3c_device_rtc);
		platform_device_register(&s3c_bd1754_device);
		platform_device_register(&s3c_bp3595_device);
		platform_device_register(&s3c_dmard03_device);
		platform_device_register(&s3c_l3g4200d_device);
		i2c_register_board_info(0, &ak4183_i2c_dev, 1);
		i2c_register_board_info(0, &i2c_device_dmard03, 1);
		i2c_register_board_info(0, &i2c_device_l3g4200d, 1);
		platform_device_register(&yup110n_led_device);
		platform_device_register(&yup160w_led_device);
		platform_device_register(&yup110_test_led);
	}
	yup150_init_pm();
}

MACHINE_START(CM31, "CM31")
	.phys_io	= S3C2410_PA_UART,
	.io_pg_offst	= (((u32)S3C24XX_VA_UART) >> 18) & 0xfffc,
	.boot_params	= S3C2410_SDRAM_PA + 0x100,

	.init_irq	= s3c24xx_init_irq,
	.map_io		= yup150_map_io,
	.init_machine	= yup150_machine_init,
	.timer		= &s3c24xx_timer,
MACHINE_END
