#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/seq_file.h>
#include <linux/spinlock.h>
#include <linux/sysctl.h>
#include <linux/wait.h>
#include <linux/bcd.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/io.h>
#include <asm/mach-types.h>

#include <mach/hardware.h>
#include <mach/regs-gpio.h>

#include <plat/regs-timer.h>
#include <mach/pwm-clock.h>

#define BD1603_MINOR	248

#define SYSTEM_REV_YUP110       0
#define SYSTEM_REV_YUP110S      1
#define SYSTEM_REV_YUP110R      2
#define SYSTEM_REV_YUP150       3
#define SYSTEM_REV_YUP110N      4
#define SYSTEM_REV_YUP150N      5
#define SYSTEM_REV_YUP110SN     6
#define SYSTEM_REV_YUP110N_LEI  7

static int bd1603_open_cnt = 0;
static unsigned int bd1603_level = 5;

static void bd1603_suspend_init(void)
{
	s3c2410_gpio_cfgpin(S3C2410_GPB0, S3C2410_GPB0_OUTP );
	s3c2410_gpio_pullup(S3C2410_GPB0, S3C24XX_GPIO_PULL_UP);
	s3c2410_gpio_setpin(S3C2410_GPB0,0);
}

static void bd1603_pwm_init(void)
{
	unsigned long tcfg0,tcfg1;

	s3c2410_gpio_cfgpin(S3C2410_GPB0, S3C2410_GPB0_TOUT0);
	s3c2410_gpio_pullup(S3C2410_GPB0, S3C24XX_GPIO_PULL_UP);

	tcfg0 = __raw_readl(S3C2410_TCFG0);
	tcfg1 = __raw_readl(S3C2410_TCFG1);
	
	tcfg0 &= (~S3C2410_TCFG_PRESCALER0_MASK);
//	tcfg0 |= 0<<0;
	tcfg1 &= (~ S3C2410_TCFG1_MUX0_MASK);
	tcfg1 |= S3C2410_TCFG1_MUX0_DIV2;
	
	__raw_writel(tcfg0, S3C2410_TCFG0);
	__raw_writel(tcfg1, S3C2410_TCFG1);
	
}

static void bd1603_set_level(unsigned int level)
{
	unsigned long tcon=0;
	if(level>16499){
		s3c2410_gpio_cfgpin(S3C2410_GPB0, S3C2410_GPB0_OUTP );
		s3c2410_gpio_pullup(S3C2410_GPB0, S3C24XX_GPIO_PULL_UP);
		s3c2410_gpio_setpin(S3C2410_GPB0,1);
	}else {
		local_irq_disable();
		tcon  = __raw_readl(S3C2410_TCON);
		
		__raw_writel(16500,  S3C2410_TCNTB(0));
		__raw_writel(level,  S3C2410_TCMPB(0));
		
		
		tcon |= S3C2410_TCON_T0MANUALUPD | S3C2410_TCON_T0RELOAD;	
		__raw_writel(tcon,  S3C2410_TCON);
		
		tcon &=(~S3C2410_TCON_T0MANUALUPD) ;	
		__raw_writel(tcon,  S3C2410_TCON);
		
		
		tcon |= S3C2410_TCON_T0START ;
		__raw_writel(tcon,  S3C2410_TCON);
		
		local_irq_enable();
	}
}

static ssize_t
bd1603_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
	unsigned int level=0;
	
	if (copy_from_user(&level, buf, 1))
		return -EFAULT;
	if (level > 52){
	    	if(machine_is_yup150()){	
		    	if((system_rev == SYSTEM_REV_YUP110N) || (system_rev == SYSTEM_REV_YUP110N_LEI))
				level=16499;
			else
			    	level=10509;	
		}
	    	if(machine_is_cm31())	
				level=16499;

		bd1603_pwm_init();
	} else if (level < 1){
		level = 0;
		bd1603_level = level;
		bd1603_suspend_init();
		return 1;
	} else {
	    	if(machine_is_yup150()){	
		    	if((system_rev == SYSTEM_REV_YUP110N) || (system_rev == SYSTEM_REV_YUP110N_LEI))
			    	level=317*level;
			else 
			    	level=202*level;
		}
	    	if(machine_is_cm31())	
		    	level=317*level;

		bd1603_pwm_init();
	}
	bd1603_level = level;
	bd1603_set_level(level);
	return 1;
}

static int
bd1603_open(struct inode *inode, struct file *file)
{
	if (bd1603_open_cnt)
		return -EBUSY;
	bd1603_open_cnt++;
	return 0;
}

static int
bd1603_release(struct inode *inode, struct file *file)
{
	bd1603_open_cnt--;
	return 0;
}

static const struct file_operations bd1603_fops = {
	.owner		= THIS_MODULE,
	.write		= bd1603_write,
	.open		= bd1603_open,
	.release	= bd1603_release,
};

static struct miscdevice bd1603_dev = {
	BD1603_MINOR,
	"bd1603",
	&bd1603_fops
};

static int __init bd1603_probe(struct platform_device *pdev)
{
	int ret;
	bd1603_pwm_init();
	ret = misc_register(&bd1603_dev);
	if (ret) {
		printk(KERN_ERR "bd1603: can't misc_register on minor=%d\n",
		       BD1603_MINOR);
		goto out;
	}
	ret = 0;
	printk(KERN_INFO "BD1603 brightness control driver\n");
      out:
	return ret;
}

static int __exit bd1603_remove(struct platform_device *pdev)
{
	misc_deregister(&bd1603_dev);
	return 0;
}

#ifdef CONFIG_PM
static int bd1603_suspend(struct platform_device *pdev, pm_message_t state)
{
	bd1603_suspend_init();
	return 0;
}

static int bd1603_resume(struct platform_device *pdev)
{
	bd1603_pwm_init();
	bd1603_set_level(bd1603_level);
	return 0;
}
#else
#define bd1603_suspend NULL
#define bd1603_resume  NULL
#endif

static struct platform_driver bd1603_driver = {
	.probe		= bd1603_probe,
	.remove		= __exit_p(bd1603_remove),
	.suspend	= bd1603_suspend,
	.resume		= bd1603_resume,
	.driver		= {
		.name	= "bd1603",
		.owner	= THIS_MODULE,
	},
};

static int __init bd1603_init(void)
{
	return platform_driver_register(& bd1603_driver);
}

static void __exit bd1603_exit(void)
{
	platform_driver_unregister(& bd1603_driver);
}

module_init(bd1603_init);
module_exit(bd1603_exit);

MODULE_LICENSE("GPL");
