#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/seq_file.h>
#include <linux/spinlock.h>
#include <linux/sysctl.h>
#include <linux/wait.h>
#include <linux/bcd.h>
#include <linux/delay.h>
#include <linux/platform_device.h>

#include <mach/hardware.h>
#include <mach/regs-gpio.h>

#define BD1754_MINOR	242

static int bd1754_open_cnt = 0;
static unsigned char bd1754_level = 40;

static void bd1754_set_level(unsigned char level)
{
	int i;

	s3c2410_gpio_cfgpin(S3C2410_GPB0, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPB0, 0);
	s3c2410_gpio_pullup(S3C2410_GPB0, S3C24XX_GPIO_PULL_UP);

	msleep(2);
	if(level==0){
	    	s3c2410_gpio_setpin(S3C2410_GPB0,0);
		msleep(1);
	} else {
	    	level = 58 - level;
		local_irq_disable();
		for (i = 0; i <= level; i++) {
		    	if (i) {
			    	s3c2410_gpio_setpin(S3C2410_GPB0,0);
				udelay(2);
			}
			s3c2410_gpio_setpin(S3C2410_GPB0,1);
			udelay(2);
		}
		local_irq_enable();
	}
}

static ssize_t
bd1754_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
	unsigned char level;

	if (copy_from_user(&level, buf, 1))
		return -EFAULT;
	/* yup120N max 57 count, backlight driver max 64 count */	
	if (level > 57) 
		level = 57;
	else if (level && level <= 2)
		level += 57;

	bd1754_level = level;
	bd1754_set_level(level);

	return 1;
}

static int
bd1754_open(struct inode *inode, struct file *file)
{
	if (bd1754_open_cnt)
		return -EBUSY;
	bd1754_open_cnt++;
	return 0;
}

static int
bd1754_release(struct inode *inode, struct file *file)
{
	bd1754_open_cnt--;
	return 0;
}

static const struct file_operations bd1754_fops = {
	.owner		= THIS_MODULE,
	.write		= bd1754_write,
	.open		= bd1754_open,
	.release	= bd1754_release,
};

static struct miscdevice bd1754_dev = {
	BD1754_MINOR,
	"BD1754",
	&bd1754_fops
};

static int __init bd1754_probe(struct platform_device *pdev)
{
	int ret;

	ret = misc_register(&bd1754_dev);
	if (ret) {
		printk(KERN_ERR "bd1754: can't misc_register on minor=%d\n",
		       BD1754_MINOR);
		goto out;
	}
	ret = 0;
	printk(KERN_INFO "BD1754 brightness control driver\n");
      out:
	return ret;
}

static int __exit bd1754_remove(struct platform_device *pdev)
{
	misc_deregister(&bd1754_dev);
	return 0;
}

#ifdef CONFIG_PM
static int bd1754_suspend(struct platform_device *pdev, pm_message_t state)
{
	bd1754_set_level(0);
	return 0;
}

static int bd1754_resume(struct platform_device *pdev)
{
	bd1754_set_level(bd1754_level);
	return 0;
}
#else
#define bd1754_suspend NULL
#define bd1754_resume  NULL
#endif

static struct platform_driver bd1754_driver = {
	.probe		= bd1754_probe,
	.remove		= __exit_p(bd1754_remove),
	.suspend	= bd1754_suspend,
	.resume		= bd1754_resume,
	.driver		= {
		.name	= "bd1754",
		.owner	= THIS_MODULE,
	},
};

static int __init bd1754_init(void)
{
	return platform_driver_register(&bd1754_driver);
}

static void __exit bd1754_exit(void)
{
	platform_driver_unregister(&bd1754_driver);
}

module_init(bd1754_init);
module_exit(bd1754_exit);

MODULE_LICENSE("GPL");
