/*
 *      YUP160W BP3595 power control 
 */

#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/seq_file.h>
#include <linux/spinlock.h>
#include <linux/sysctl.h>
#include <linux/wait.h>
#include <linux/bcd.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/io.h>
#include <linux/hrtimer.h>
#include <linux/ktime.h>
#include <asm/mach-types.h>

#include <plat/regs-timer.h>
#include <mach/pwm-clock.h>
#include <mach/hardware.h>
#include <mach/regs-gpio.h>
#include <linux/mmc/sdhci.h>
#include <plat/sdhci.h>
#include <plat/devs.h>

#include <linux/kobject.h>
#define BP3595_MINOR	241	

struct timer_list wifi ;
static int bp3595_open_cnt = 0;

static ssize_t
bp3595_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
    	struct sdhci_host *host = platform_get_drvdata(&s3c_device_hsmmc0);
	unsigned char 	onoff;
	unsigned long 	flags;
	int		err;

	err = copy_from_user(&onoff, buf, 1);
	if (err < 0)
		return err;
	if(onoff == 1 && host->wifi_attach == 0){
	    	if(host){
		    	s3c2410_gpio_setpin(S3C2410_GPB6, 1);
			s3c2410_gpio_pullup(S3C2410_GPB6, S3C24XX_GPIO_PULL_UP);
			msleep(20);
			spin_lock_irqsave(&host->lock, flags);
			host->flags &= ~SDHCI_DEVICE_DEAD;
			host->quirks |= SDHCI_QUIRK_BROKEN_CARD_DETECTION;
			tasklet_schedule(&host->card_tasklet);
			spin_unlock_irqrestore(&host->lock, flags);
		}
	} 
	if(onoff == 0 ){
	       	if(host){
		    	spin_lock_irqsave(&host->lock, flags);
		    	host->flags |= SDHCI_DEVICE_DEAD;
			host->quirks &= ~SDHCI_QUIRK_BROKEN_CARD_DETECTION;
			tasklet_schedule(&host->card_tasklet);
			spin_unlock_irqrestore(&host->lock, flags);
			s3c2410_gpio_setpin(S3C2410_GPB6, 0);
			s3c2410_gpio_pullup(S3C2410_GPB6, S3C24XX_GPIO_PULL_DOWN);
			msleep(20);		
		}
	}
	return 1;
}
static ssize_t
bp3595_read(struct file *file,  char __user *buf, size_t count, loff_t *ppos)
{
    	struct sdhci_host *host = platform_get_drvdata(&s3c_device_hsmmc0);
	static int wifi_status;
	int err;
	
	wifi_status = host->wifi_attach;
	err = copy_to_user(buf, &wifi_status, 1);
	if(err<0)
	    	return err;
	return 1;
}

static int
bp3595_open(struct inode *inode, struct file *file)
{
	if (bp3595_open_cnt)
		return -EBUSY;
	bp3595_open_cnt++;
	return 0;
}

static int
bp3595_release(struct inode *inode, struct file *file)
{
	bp3595_open_cnt--;
	return 0;
}

static const struct file_operations bp3595_fops = {
	.owner		= THIS_MODULE,
	.write		= bp3595_write,
	.read		= bp3595_read,
	.open		= bp3595_open,
	.release	= bp3595_release,
};

static struct miscdevice bp3595_dev = {
	BP3595_MINOR,
	"bp3595",
	&bp3595_fops
};
static int wlan_pw = 0;
static ssize_t wlan_show(struct kobject *kobj,
	                          struct kobj_attribute *attr, char *buf)
{
    	struct sdhci_host *host = platform_get_drvdata(&s3c_device_hsmmc0);
	wlan_pw = host->wifi_attach;
    	return sprintf(buf, "%d\n", wlan_pw);
}
static ssize_t wlan_store(struct kobject *kobj,
	                         struct kobj_attribute *attr, const char * buf, size_t n)
{
    	 struct sdhci_host *host = platform_get_drvdata(&s3c_device_hsmmc0);
	 unsigned long   flags;

    	int value;
        if (sscanf(buf, "%d", &value) != 1 ||
		            (value != 0 && value != 1))
	{
	    	printk(KERN_ERR "wlan_store: Invalid value\n");
		return -EINVAL;
	} 
	if (wlan_pw == value)return n;
	wlan_pw = value;
	if(wlan_pw == 1 && host->wifi_attach == 0){
	    	if(host){
		    	s3c2410_gpio_setpin(S3C2410_GPB6, 1);
			s3c2410_gpio_pullup(S3C2410_GPB6, S3C24XX_GPIO_PULL_UP);
			msleep(20);
			spin_lock_irqsave(&host->lock, flags);
			host->flags &= ~SDHCI_DEVICE_DEAD;
			host->quirks |= SDHCI_QUIRK_BROKEN_CARD_DETECTION;
			tasklet_schedule(&host->card_tasklet);
			spin_unlock_irqrestore(&host->lock, flags);
		}
	}
	if(wlan_pw == 0 ){
	    	if(host){
		    	spin_lock_irqsave(&host->lock, flags);
			host->flags |= SDHCI_DEVICE_DEAD;
			host->quirks &= ~SDHCI_QUIRK_BROKEN_CARD_DETECTION;
			tasklet_schedule(&host->card_tasklet);
			spin_unlock_irqrestore(&host->lock, flags);
			s3c2410_gpio_setpin(S3C2410_GPB6, 0);
			s3c2410_gpio_pullup(S3C2410_GPB6, S3C24XX_GPIO_PULL_DOWN);
			msleep(20);
		}
	}
	return n;
}
static struct kobj_attribute wlan_attr =
        __ATTR(wlan, 0644, wlan_show, wlan_store);


static int __init bp3595_probe(struct platform_device *pdev)
{
	int ret;
	/* YUP160W bp3595 power ctrl */

	__raw_writel(0,  S3C24XX_GPIO_BASE(S3C2410_GPB6) + 0x0c);
	s3c2410_gpio_cfgpin(S3C2410_GPB6, S3C2410_GPB6_OUTP);
	s3c2410_gpio_setpin(S3C2410_GPB6, 1);
	s3c2410_gpio_pullup(S3C2410_GPB6, S3C24XX_GPIO_PULL_UP);
	
	ret = misc_register(&bp3595_dev);
	if (ret) {
		printk(KERN_ERR "bp3595: can't misc_register on minor=%d\n",
		       BP3595_MINOR);
		goto out;
	}
	ret = 0;
	sysfs_create_file(power_kobj, &wlan_attr.attr);
	printk(KERN_INFO "YUP160W bp3595 wlan power control driver\n");
      out:
	return ret;
}

static int __exit bp3595_remove(struct platform_device *pdev)
{
	misc_deregister(&bp3595_dev);
	return 0;
}

#ifdef CONFIG_PM
static int bp3595_suspend(struct platform_device *pdev, pm_message_t state)
{
	return 0;
}

static int bp3595_resume(struct platform_device *pdev)
{
	return 0;
}
#else
#define bp3595_suspend NULL
#define bp3595_resume  NULL
#endif

static struct platform_driver bp3595_driver = {
	.probe		= bp3595_probe,
	.remove		= __exit_p(bp3595_remove),
	.suspend	= bp3595_suspend,
	.resume		= bp3595_resume,
	.driver		= {
		.name	= "bp3595",
		.owner	= THIS_MODULE,
	},
};

static int __init bp3595_init(void)
{
	return platform_driver_register(&bp3595_driver);
}

static void __exit bp3595_exit(void)
{
	platform_driver_unregister(&bp3595_driver);
}


module_init(bp3595_init);
module_exit(bp3595_exit);

MODULE_LICENSE("GPL");
