#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/seq_file.h>
#include <linux/spinlock.h>
#include <linux/sysctl.h>
#include <linux/wait.h>
#include <linux/bcd.h>
#include <linux/delay.h>
#include <linux/platform_device.h>

#include <linux/i2c.h>
#include <linux/gpio.h>


#include <linux/dmard03.h>

#include <asm/uaccess.h>

#define DMARD03_MINOR	247

/* CONTROL REGISTERS */
#define CTRL_REG1               0x08
#define CTRL_REG2               0x09
#define CTRL_REG3               0x0A
#define CTRL_REG4               0x0B

/* INTERRUPT STATUS REGISTERS*/
#define INT_STATUS              0x0C


#define I2C_RETRY_DELAY         5
#define I2C_RETRIES             5

/* RESUME STATE INDICES */
#define RES_CTRL_REG1           0
#define RES_CTRL_REG2           1
#define RES_CTRL_REG3           2
#define RES_CTRL_REG4           3

/* DATA REGISTERS */
#define TOUT_H                  0
#define TOUT_L                  0x01
#define XOUT_H                  0x02
#define XOUT_L                  0x03
#define YOUT_H                  0x04
#define YOUT_L                  0x05
#define ZOUT_H                  0x06
#define ZOUT_L                  0x07

static int dmard03_open_cnt = 0;
struct dmard03_data {
	struct i2c_client *client;
	struct dmard03_platform_data *pdata;
	int hw_initialized;
	u8 resume_state[5];

};


static struct dmard03_data *d03;
static int dmard03_i2c_read(struct dmard03_data *d03, u8 *buf, int len)
{
	int err;
	int tries = 0;
	
	struct i2c_msg msgs[] = {
		{
			.addr = d03->client->addr,
			.flags = d03->client->flags & I2C_M_TEN,
			.len = 1,
			.buf = buf,
		},
		{
			.addr = d03->client->addr,
			.flags = (d03->client->flags & I2C_M_TEN) | I2C_M_RD,
			.len = len,
			.buf = buf,
		},
	};
	do {
		err = i2c_transfer(d03->client->adapter, msgs, 2);
		if (err != 2)
			msleep_interruptible(I2C_RETRY_DELAY);
	} while ((err != 2) && (++tries < I2C_RETRIES));
	if (err != 2) {
		dev_err(&d03->client->dev, "read transfer error\n");
		err = -EIO;
	} else {
		err = 0;
	}
	
	return err;
}

static int dmard03_i2c_write(struct dmard03_data *d03, u8 * buf, int len)
{
	int err;
	int tries = 0;
	
	struct i2c_msg msgs[] = {
		{
			.addr = d03->client->addr,
			.flags = d03->client->flags & I2C_M_TEN,
			.len = len + 1,
			.buf = buf,
		},
	};
	do {
		err = i2c_transfer(d03->client->adapter, msgs, 1);
		if (err != 1)
			msleep_interruptible(I2C_RETRY_DELAY);
	} while ((err != 1) && (++tries < I2C_RETRIES));
	if (err != 1) {
		dev_err(&d03->client->dev, "write transfer error\n");
		err = -EIO;
	} else {
		err = 0;
	}
	return err;
}

static int dmard03_hw_init(struct dmard03_data *d03)
{
	int err = -1;	
	u8 buf[2];
	

	buf[0] = CTRL_REG1;
	buf[1] = d03->resume_state[RES_CTRL_REG1];
	err = dmard03_i2c_write(d03, buf, 1);
	if (err < 0)
		return err;
	buf[0] = CTRL_REG2;
	buf[1] = d03->resume_state[RES_CTRL_REG2];
	err = dmard03_i2c_write(d03, buf, 1);
	if (err < 0)
		return err;
	buf[0] = CTRL_REG3;
	buf[1] = d03->resume_state[RES_CTRL_REG3];
	err = dmard03_i2c_write(d03, buf, 1);
	if (err < 0)
		return err;
	buf[0] = CTRL_REG4;
	buf[1] = d03->resume_state[RES_CTRL_REG4];
	err = dmard03_i2c_write(d03, buf, 1);
	if (err < 0)
		return err;
	
	d03->hw_initialized = 1;
	return 0;
}




static ssize_t
dmard03_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
	return 0;
}
static ssize_t
dmard03_read(struct file *file, char __user *buf, size_t count, loff_t *ppos)
{
	
	int err = -1;
        /* Data bytes from hardware x, y, z */
        u8 acc_data[6];
	/* x,y,z hardware values */
	short hw_d[3] = { 0 };
	
	acc_data[0] = XOUT_H ;
	err = dmard03_i2c_read(d03, acc_data,sizeof(u8)*6);
	if (err < 0)
		return err;
	hw_d[0] =(short)acc_data[0];
	hw_d[0] = (hw_d[0]<<3) | ((short)acc_data[1] & 0x7);
	hw_d[1] =(short)acc_data[2];
	hw_d[1] = (hw_d[1]<<3) | ((short)acc_data[3] & 0x7);
	hw_d[2] =(short)acc_data[4];
	hw_d[2] = (hw_d[2]<<3) | ((short)acc_data[5] & 0x7);
	
	if(copy_to_user(buf,hw_d,count))
		return -1;
	return 0;
}

static int dmard03_device_power_on(struct dmard03_data *d03)
{
	int err;
	
	if (!d03->hw_initialized) {
		mdelay(110);
		err = dmard03_hw_init(d03);
		if (err < 0) {
			d03->hw_initialized=0;
			return err;
		}
	}
	return 0;
}


static int
dmard03_open(struct inode *inode, struct file *file)
{
	if (dmard03_open_cnt)
		return -EBUSY;
	dmard03_open_cnt++;
	return 0;
}

static int
dmard03_release(struct inode *inode, struct file *file)
{
	dmard03_open_cnt--;
	return 0;
}

static const struct file_operations dmard03_fops = {
	.owner		= THIS_MODULE,
	.write		= dmard03_write,
	.read		= dmard03_read,
	.open		= dmard03_open,
	.release	= dmard03_release,
};

static struct miscdevice dmard03_dev = {
	DMARD03_MINOR,
	"dmard03",
	&dmard03_fops
};


static int dmard03_probe(struct i2c_client *client,
		                           const struct i2c_device_id *id)
{
	int err=-1;
	if (client->dev.platform_data == NULL) {
                dev_err(&client->dev, "platform data is NULL. exiting.\n");
                err = -ENODEV;
                goto err0;
        }
        if (!i2c_check_functionality(client->adapter, I2C_FUNC_I2C)) {
                dev_err(&client->dev, "client not i2c capable\n");
                err = -ENODEV;
                goto err0;
        }
	d03 = kzalloc(sizeof(*d03), GFP_KERNEL);
        if (d03 == NULL) {
		dev_err(&client->dev,
				"failed to allocate memory for module data\n");
		err = -ENOMEM;
		goto err0;
	}
	memset(d03, 0, sizeof(*d03));	

	d03->client = client;	
	d03->pdata = kmalloc(sizeof(*d03->pdata), GFP_KERNEL);
	if (d03->pdata == NULL)
		goto err1;
	memcpy(d03->pdata, client->dev.platform_data, sizeof(*d03->pdata));
	i2c_set_clientdata(client, d03);
	if (d03->pdata->init) {
		err = d03->pdata->init();
		if (err < 0)
			goto err2;
	}

	memset(d03->resume_state, 0, ARRAY_SIZE(d03->resume_state));
	d03->resume_state[RES_CTRL_REG1]    = d03->pdata->ctrl_reg1_init;
        d03->resume_state[RES_CTRL_REG2]    = d03->pdata->ctrl_reg2_init;
        d03->resume_state[RES_CTRL_REG3]    = d03->pdata->ctrl_reg3_init;
        d03->resume_state[RES_CTRL_REG4]    = d03->pdata->ctrl_reg4_init;
	
	err = dmard03_device_power_on(d03);
        if (err < 0)
		                goto err3;

	err = misc_register(&dmard03_dev);
	if (err) {
		printk(KERN_ERR "accelerometer: can't misc_register on minor=%d\n",
				DMARD03_MINOR);
		goto out;
	}
	dev_info(&client->dev, "dmard03 probed\n");
	return 0;

err3:
	if (d03->pdata->exit)
                d03->pdata->exit();
err2:
	kfree(d03->pdata);
err1:
	kfree(d03);
err0:
out:
	return err;
}

static int __devexit dmard03_remove(struct i2c_client *client)
{
	struct dmard03_data *d03 = i2c_get_clientdata(client);
	d03->hw_initialized=0;
 	kfree(d03->pdata);
  	kfree(d03);
	misc_deregister(&dmard03_dev);
	return 0;
}

#define dmard03_suspend NULL
#define dmard03_resume  NULL

static const struct i2c_device_id dmard03_id[] = {
	        {"dmard03", 0},
		        {},
};

MODULE_DEVICE_TABLE(i2c, dmard03_id);

static struct i2c_driver dmard03_driver = {
	        .driver = {
			.name = "dmard03",
		},
		.probe = dmard03_probe,
		.remove = __devexit_p(dmard03_remove),
		.resume = dmard03_resume,
		.suspend = dmard03_suspend,
		.id_table = dmard03_id,
};

static int __init dmard03_init(void)
{
	        pr_info(KERN_INFO "DMARD03 accelerometer driver init\n");
		return i2c_add_driver(&dmard03_driver);
}

static void __exit dmard03_exit(void)
{
	        i2c_del_driver(&dmard03_driver);
		return;
}

module_init(dmard03_init);
module_exit(dmard03_exit);

MODULE_DESCRIPTION("DMARD03 accelerometer driver");
MODULE_AUTHOR("Asiteq");
MODULE_LICENSE("GPL");
      
#if 0
static struct platform_driver idg500_driver = {
	.probe          = idg500_probe,
	.remove         = __exit_p(idg500_remove),
	.suspend        = idg500_suspend,
	.resume         = idg500_resume,
	.driver         = {
		.name   = "idg500",
		.owner  = THIS_MODULE,
	},
};

static int __init idg500_init(void)
{
	        return platform_driver_register(&idg500_driver);
}

static void __exit idg500_exit(void)
{
	        platform_driver_unregister(&idg500_driver);
}

module_init(idg500_init);
module_exit(idg500_exit);

MODULE_LICENSE("GPL");

#endif


                                                
