#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/seq_file.h>
#include <linux/spinlock.h>
#include <linux/sysctl.h>
#include <linux/wait.h>
#include <linux/bcd.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
//#include <linux/io.h>
//#include <linux/clk.h>

#include <mach/hardware.h>
#include <mach/regs-gpio.h>

#include <plat/adc.h>
#include <asm/uaccess.h>

#define IDG500_MINOR	245

static struct s3c_adc_client    *adc_client;
static int idg500_open_cnt = 0;

static ssize_t
idg500_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
	s3c2410_gpio_setpin(S3C2410_GPG3, 1);
	udelay(50);
	s3c2410_gpio_setpin(S3C2410_GPG3, 0);
	msleep(7);
	return 0;
}
static ssize_t
idg500_read(struct file *file, char __user *buf, size_t count, loff_t *ppos)
{
	int tmp[2]={0};	
	tmp[0]=s3c_adc_read(adc_client, 6);//ADC 6
	tmp[1]=s3c_adc_read(adc_client, 7);//ADC 7
	copy_to_user(buf,tmp,count);
	//printk(KERN_INFO "idg500 Xout :%d\n",tmp[0]);
	//printk(KERN_INFO "idg500 Yout :%d\n",tmp[1]);
	return 0;
}
static int
idg500_open(struct inode *inode, struct file *file)
{
	if (idg500_open_cnt)
		return -EBUSY;
	idg500_open_cnt++;
	return 0;
}

static int
idg500_release(struct inode *inode, struct file *file)
{
	idg500_open_cnt--;
	return 0;
}

static const struct file_operations idg500_fops = {
	.owner		= THIS_MODULE,
	.write		= idg500_write,
	.read		= idg500_read,
	.open		= idg500_open,
	.release	= idg500_release,
};

static struct miscdevice idg500_dev = {
	IDG500_MINOR,
	"idg500",
	&idg500_fops
};


static int __init idg500_probe(struct platform_device *pdev)
{
	int ret;
	
	s3c2410_gpio_cfgpin(S3C2410_GPG3, S3C2410_GPIO_OUTPUT);
        s3c2410_gpio_setpin(S3C2410_GPG3, 0);
	s3c2410_gpio_pullup(S3C2410_GPG3, S3C24XX_GPIO_PULL_UP);

	adc_client = s3c_adc_register(pdev, NULL, NULL, 0);
	if (!adc_client) {
		dev_err(&pdev->dev, "cannot register adc\n");
	}

	ret = misc_register(&idg500_dev);
	if (ret) {
		printk(KERN_ERR "gyro_adc: can't misc_register on minor=%d\n",
				IDG500_MINOR);
		goto out;
	}
	ret = 0;
	printk(KERN_INFO "gyro_adc TRG control driver\n");
out:
	return ret;
}

static int __exit idg500_remove(struct platform_device *pdev)
{
	s3c_adc_release(adc_client);
	misc_deregister(&idg500_dev);
	return 0;
}

#define idg500_suspend NULL
#define idg500_resume  NULL

static struct platform_driver idg500_driver = {
	.probe          = idg500_probe,
	.remove         = __exit_p(idg500_remove),
	.suspend        = idg500_suspend,
	.resume         = idg500_resume,
	.driver         = {
		.name   = "idg500",
		.owner  = THIS_MODULE,
	},
};

static int __init idg500_init(void)
{
	        return platform_driver_register(&idg500_driver);
}

static void __exit idg500_exit(void)
{
	        platform_driver_unregister(&idg500_driver);
}

module_init(idg500_init);
module_exit(idg500_exit);

MODULE_LICENSE("GPL");
