/*
 *      YUP120N LED control & AUDIO_LINE SC_MUTE control
 *      
 *
 *      tmp[0] : 0=> LED0, 1=> LED1, 2=>LED2, 3=>LED3 4=>all LED bright control
 *      tme[1] : 0: red,  1: green, 2: blue 
 *      tmp[2] : value 
 *	tmp[3] : all LED bright control val
 */

#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/seq_file.h>
#include <linux/spinlock.h>
#include <linux/sysctl.h>
#include <linux/wait.h>
#include <linux/bcd.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/io.h>
#include <linux/hrtimer.h>
#include <linux/ktime.h>
#include <asm/mach-types.h>


#include <plat/regs-timer.h>
#include <mach/pwm-clock.h>
#include <mach/hardware.h>
#include <mach/regs-gpio.h>

#define MS_TO_NS(x)	(x * 1E6L)
#define LED_MINOR	240

#define PORT1 		0	
#define PORT2 		1
#define PORT3 		2	
#define PORT4 		3	

#define RED		0
#define GREEN		1
#define BLUE		2

static struct hrtimer hr_timer;
static ktime_t ktime;
static int led_open_cnt = 0;
static unsigned int timercount=0;
static unsigned char rgb_val[12];
enum hrtimer_restart rgb_led_hrtimer_callback(struct hrtimer *timer)
{
    	unsigned char i;
	if(timercount>30){
	    	timercount = 0;
	}
	for(i=0;i<3;i++){
	    	if(timercount < rgb_val[i])
		    	s3c2410_gpio_setpin(S3C2410_GPIONO(S3C2410_GPIO_BANKA, i),1); 
		else 
			s3c2410_gpio_setpin(S3C2410_GPIONO(S3C2410_GPIO_BANKA, i),0);
	}
    	timercount++;
	hrtimer_forward_now(timer,ktime);
	
	return HRTIMER_RESTART;
}
static void led_pwm_init(void)
{
    	unsigned long tcfg0,tcfg1;
	s3c2410_gpio_cfgpin(S3C2410_GPB1, S3C2410_GPB1_TOUT1);
	s3c2410_gpio_pullup(S3C2410_GPB1, S3C24XX_GPIO_PULL_UP);
	
	tcfg0 = __raw_readl(S3C2410_TCFG0);
	tcfg1 = __raw_readl(S3C2410_TCFG1);
	
	tcfg0 &= (~S3C2410_TCFG_PRESCALER0_MASK);
	tcfg1 &= (~ S3C2410_TCFG1_MUX1_MASK);
	tcfg1 |= S3C2410_TCFG1_MUX1_DIV2;
	
	__raw_writel(tcfg0, S3C2410_TCFG0);
	__raw_writel(tcfg1, S3C2410_TCFG1);

}
static void led_bright_setting(unsigned int level)
{	
    	unsigned long tcon=0;
	local_irq_disable();
	tcon  = __raw_readl(S3C2410_TCON);
	if(level>2350)
	    level = 2399;
	__raw_writel(2400,  S3C2410_TCNTB(1));
	__raw_writel(level,  S3C2410_TCMPB(1));
	
	
	tcon |= S3C2410_TCON_T1MANUALUPD | S3C2410_TCON_T1RELOAD;
	__raw_writel(tcon,  S3C2410_TCON);
	
	tcon &=(~S3C2410_TCON_T1MANUALUPD) ;
	__raw_writel(tcon,  S3C2410_TCON);
	
	
	tcon |= S3C2410_TCON_T1START ;
	__raw_writel(tcon,  S3C2410_TCON);
	
	local_irq_enable();
}
static void led_rgb(unsigned int port,unsigned int rgb ,unsigned int val)
{
    	switch(rgb){
	    	case GREEN :	rgb_val[0] = val;
				break;
		case BLUE :	rgb_val[1] = val;
				break;
		case RED :	rgb_val[2] = val;
				break;
	}
}


static ssize_t
led_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
	unsigned int level[4];

	if (copy_from_user(&level, buf, 16))
		return -EFAULT;
	
	switch (level[0]) {
	    	case 0 : led_rgb(0,level[1],level[2]);
			 break;
		case 1 : led_rgb(1,level[1],level[2]);
			 break;
		case 2 : led_rgb(2,level[1],level[2]);
			 break;
		case 3 : led_rgb(3,level[1],level[2]); 
			 break;
		case 4 : led_bright_setting(level[3]);
			 break;
		defult : return -EFAULT;
	}
	return 1;
}

static int
led_open(struct inode *inode, struct file *file)
{
	if (led_open_cnt)
		return -EBUSY;
	led_open_cnt++;
	return 0;
}

static int
led_release(struct inode *inode, struct file *file)
{
	led_open_cnt--;
	return 0;
}

static const struct file_operations led_fops = {
	.owner		= THIS_MODULE,
	.write		= led_write,
	.open		= led_open,
	.release	= led_release,
};

static struct miscdevice led_dev = {
	LED_MINOR,
	"yup160w_led",
	&led_fops
};

static int __init led_probe(struct platform_device *pdev)
{
	int ret,i;
	/* YUP160W RGB LED ctrl */
	s3c2410_gpio_cfgpin(S3C2410_GPA0, S3C2410_GPA0_OUT);		
	s3c2410_gpio_cfgpin(S3C2410_GPA1, S3C2410_GPA1_OUT);
	s3c2410_gpio_cfgpin(S3C2410_GPA2, S3C2410_GPA2_OUT);
	s3c2410_gpio_cfgpin(S3C2410_GPB1, S3C2410_GPB0_OUTP );
	s3c2410_gpio_pullup(S3C2410_GPB1, S3C24XX_GPIO_PULL_UP);
	s3c2410_gpio_setpin(S3C2410_GPB1,1);

	for(i=0;i<12;i++)
	    	rgb_val[i]=0;
	
	ktime = ktime_set(0,500000);
	hrtimer_init(&hr_timer,CLOCK_MONOTONIC,HRTIMER_MODE_REL);
	hr_timer.function = &rgb_led_hrtimer_callback;
	hrtimer_start(&hr_timer, ktime, HRTIMER_MODE_REL);
	led_pwm_init();

	ret = misc_register(&led_dev);
	if (ret) {
		printk(KERN_ERR "RGB led: can't misc_register on minor=%d\n",
		       LED_MINOR);
		goto out;
	}
	ret = 0;
	printk(KERN_INFO "YUP160W RGB LED control driver\n");
      out:
	return ret;
}

static int __exit led_remove(struct platform_device *pdev)
{
	misc_deregister(&led_dev);
	return 0;
}

#ifdef CONFIG_PM
static int led_suspend(struct platform_device *pdev, pm_message_t state)
{
	return 0;
}

static int led_resume(struct platform_device *pdev)
{
	return 0;
}
#else
#define led_suspend NULL
#define led_resume  NULL
#endif

static struct platform_driver led_driver = {
	.probe		= led_probe,
	.remove		= __exit_p(led_remove),
	.suspend	= led_suspend,
	.resume		= led_resume,
	.driver		= {
		.name	= "yup160w_led",
		.owner	= THIS_MODULE,
	},
};

static int __init led_init(void)
{
	return platform_driver_register(&led_driver);
}

static void __exit led_exit(void)
{
	platform_driver_unregister(&led_driver);
}

module_init(led_init);
module_exit(led_exit);

MODULE_LICENSE("GPL");
