/*
 * Power control for Himax 
 *
 * Copyright (C) 2006, 2007 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/delay.h>
#include <linux/err.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/lcd.h>
#include <linux/module.h>
#include <linux/spi/spi.h>
/* FIXME */
#include <mach/regs-gpio.h>

#define SYSTEM_REV_YUP150       3


#define POWER_IS_ON(pwr)	((pwr) <= FB_BLANK_NORMAL)

struct eh300wq {
	struct spi_device	*spi;
	int			power;
	struct lcd_device	*ld;

	u8			device_id; /* 011101 */
};

static int hx8352_write_reg(struct eh300wq *lcd, u8 reg, u8 data)
{
	ssize_t retval;
	u8	cmd[2];

	cmd[0] = lcd->device_id << 2;
	cmd[1] = reg;
	retval = spi_write(lcd->spi, cmd, 2);
	if (retval < 0) {
		dev_err(&lcd->spi->dev, "error %d writing reg index%02X\n",retval, reg);
		return retval;
	}

	cmd[0] = (lcd->device_id << 2) | 0x2;
	cmd[1] = data;
	retval = spi_write(lcd->spi, cmd, 2);
	if (retval  < 0) {
		dev_err(&lcd->spi->dev, "error %d writing data at index%02X\n",retval, reg);
		return retval;
	}

	return 0;
}

#if 0
static int hx8352_read_reg(struct eh300wq *lcd, u8 reg, u8 *data)
{
	ssize_t retval;
	u8	cmd[3];

	cmd[0] = lcd->device_id << 2;
	cmd[1] = reg;
	retval = spi_write(lcd->spi, cmd, 2);
	if (retval < 0) {
		dev_err(&lcd->spi->dev, "error %d reading reg index%02X\n",retval, reg);
		return retval;
	}

	cmd[0] = (lcd->device_id << 2) | 0x3;
	retval = spi_write_then_read(lcd->spi, cmd, 1, cmd+1, 2);
	if (retval  < 0) {
		dev_err(&lcd->spi->dev, "error %d reading data at index%02X\n",retval, reg);
		return retval;
	}

	*data = cmd[2];
	return 0;
}
#endif

static void eh300wq_early_init(struct eh300wq *lcd)
{
	static u8 reg_vals[][2] = {
		{0x83,0X02},
		{0x85,0X03},
		{0x8B,0X00},
		{0x8C,0X93},
		{0x91,0X01},
		{0x83,0X00},
	};
	int i;

	for (i = 0; i < ARRAY_SIZE(reg_vals); i++)
		hx8352_write_reg(lcd, reg_vals[i][0], reg_vals[i][1]);
}

static void eh300wq_set_gamma(struct eh300wq *lcd)
{
	static u8 gamma_regs[][2] = {
		{0x3E,0xF0},
		{0x3F,0x07},
		{0x41,0X43},
		{0x42,0X16},
		{0x43,0X16},
		{0x44,0X43},
		{0x45,0X77},
		{0x46,0X00},
		{0x47,0X1E},
		{0x48,0X0F},
		{0x49,0X00},
	};
	int i;

	for (i = 0; i < ARRAY_SIZE(gamma_regs); i++) {
		if (system_rev == SYSTEM_REV_YUP150) {
			if (i == 0)
				continue;
		}
		hx8352_write_reg(lcd, gamma_regs[i][0], gamma_regs[i][1]);
	}
}

static void eh300wq_late_init(struct eh300wq *lcd)
{
	static u8 regs_step1[][2] = {
		/*USER SETTING*/
		{0x17,0X91},	/* OSC Control 1 Register */
		{0x23,0X01},	/* Display Control 1 Register */
		/*POWER SUPPLY SETTING*/
		{0x2B,0X6c},
		//{0x2B,0X7d},
		{0x1B,0X14},
		//{0x1B,0X44},
		{0x1A,0X13},
	//	{0x1A,0X15},
#if 1
		{0x1C,0X0D},	/* VREG1 for gamma voltage : VBGPx 4.6 */
#else
		{0x1C,0X0C},	/* VREG1 for gamma voltage : VBGPx 4.4 */
#endif
		{0x1F,0X27},
		{0x19,0X0A},
		{0x19,0X1A},
		{0x19,0X12},
		{0x1E,0X2c},
		{0x5A,0X01},	/* Digital gamma correction enable */
		/*DISPLAY ON*/
		{0x3C,0XC0},
		{0x3D,0XC0},
		{0x34,0X38},
		{0x35,0X38},
		{0x24,0X38},
	};
	static u8 regs_step2[][2] = {
		{0x24,0X3C},	/* Display Control 2 Register */
		{0x16,0X08},	/* MY:0 MX:0 GS:0 BGR:1 */
		{0x01,0X02},
		{0x55,0X00},
	};
	int i;

	for (i = 0; i < ARRAY_SIZE(regs_step1); i++)
		hx8352_write_reg(lcd, regs_step1[i][0], regs_step1[i][1]);
	msleep(50);
	for (i = 0; i < ARRAY_SIZE(regs_step2); i++)
		hx8352_write_reg(lcd, regs_step2[i][0], regs_step2[i][1]);
}

static int eh300wq_power_on(struct eh300wq *lcd)
{
	/* FIXME : reset LCD */
	s3c2410_gpio_cfgpin(S3C2410_GPB1, S3C2410_GPIO_OUTPUT);
        s3c2410_gpio_setpin(S3C2410_GPB1, 0);
        s3c2410_gpio_pullup(S3C2410_GPB1, S3C24XX_GPIO_PULL_UP);
	msleep(10);
        s3c2410_gpio_setpin(S3C2410_GPB1, 1);
	if (system_rev == SYSTEM_REV_YUP150)
		msleep(10);

	/* power on sequence */
	eh300wq_early_init(lcd);
	eh300wq_set_gamma(lcd);
	eh300wq_late_init(lcd);
	return 0;
}

static int eh300wq_power_off(struct eh300wq *lcd)
{
	/* FXIME */
	return 0;
}

static int eh300wq_power(struct eh300wq *lcd, int power)
{
        int ret = 0;

        if (POWER_IS_ON(power) && !POWER_IS_ON(lcd->power))
                ret = eh300wq_power_on(lcd);
        else if (!POWER_IS_ON(power) && POWER_IS_ON(lcd->power))
                ret = eh300wq_power_off(lcd);

        if (!ret)
                lcd->power = power;

        return ret;
}

static int eh300wq_set_power(struct lcd_device *ld, int power)
{
	struct eh300wq *lcd = lcd_get_data(ld);
	return eh300wq_power(lcd, power);
}

static int eh300wq_get_power(struct lcd_device *ld)
{
	struct eh300wq *lcd = lcd_get_data(ld);
	return lcd->power;
}

static struct lcd_ops ltv_ops = {
	.get_power	= eh300wq_get_power,
	.set_power	= eh300wq_set_power,
};

static int __devinit eh300wq_probe(struct spi_device *spi)
{
	struct eh300wq *lcd;
	struct lcd_device *ld;
	int ret;

	lcd = kzalloc(sizeof(struct eh300wq), GFP_KERNEL);
	if (!lcd)
		return -ENOMEM;

	lcd->spi = spi;
	lcd->power = FB_BLANK_POWERDOWN;
	lcd->device_id = 0x1D; /* FIXME : 011101 */

	ld = lcd_device_register("eh300wq", &spi->dev, lcd, &ltv_ops);
	if (IS_ERR(ld)) {
		ret = PTR_ERR(ld);
		goto out_free_lcd;
	}
	lcd->ld = ld;

	ret = eh300wq_power(lcd, FB_BLANK_UNBLANK);
	if (ret)
		goto out_unregister;

	dev_set_drvdata(&spi->dev, lcd);

	return 0;

out_unregister:
	lcd_device_unregister(ld);
out_free_lcd:
	kfree(lcd);
	return ret;
}

static int __devexit eh300wq_remove(struct spi_device *spi)
{
	struct eh300wq *lcd = dev_get_drvdata(&spi->dev);

	eh300wq_power(lcd, FB_BLANK_POWERDOWN);
	lcd_device_unregister(lcd->ld);
	kfree(lcd);

	return 0;
}

#ifdef CONFIG_PM
static int eh300wq_suspend(struct spi_device *spi, pm_message_t state)
{
	struct eh300wq *lcd = dev_get_drvdata(&spi->dev);

	return eh300wq_power(lcd, FB_BLANK_POWERDOWN);
}

static int eh300wq_resume(struct spi_device *spi)
{
	struct eh300wq *lcd = dev_get_drvdata(&spi->dev);

	return eh300wq_power(lcd, FB_BLANK_UNBLANK);
}
#else
#define eh300wq_suspend	NULL
#define eh300wq_resume		NULL
#endif

/* Power down all displays on reboot, poweroff or halt */
static void eh300wq_shutdown(struct spi_device *spi)
{
	struct eh300wq *lcd = dev_get_drvdata(&spi->dev);

	eh300wq_power(lcd, FB_BLANK_POWERDOWN);
}

static struct spi_driver eh300wq_driver = {
	.driver = {
		.name		= "eh300wq",
		.bus		= &spi_bus_type,
		.owner		= THIS_MODULE,
	},

	.probe		= eh300wq_probe,
	.remove		= __devexit_p(eh300wq_remove),
	.shutdown	= eh300wq_shutdown,
	.suspend	= eh300wq_suspend,
	.resume		= eh300wq_resume,
};

static int __init eh300wq_init(void)
{
	return spi_register_driver(&eh300wq_driver);
}

static void __exit eh300wq_exit(void)
{
	spi_unregister_driver(&eh300wq_driver);
}
module_init(eh300wq_init);
module_exit(eh300wq_exit);

MODULE_LICENSE("GPL");
