/*
 * Power control for Himax 
 *
 * Copyright (C) 2006, 2007 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/delay.h>
#include <linux/err.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/lcd.h>
#include <linux/module.h>
#include <linux/io.h>
/* FIXME */
#include <plat/regs-fb.h>
#include <mach/regs-gpio.h>

#define         ENABLE      1
#define         DISABLE     0
#define         INDEX       0
#define         DATA        1


#define POWER_IS_ON(pwr)	((pwr) <= FB_BLANK_NORMAL)

struct l10tlm012 {
	struct lcd_device	*ld;
	int			power;
};


static void __iomem             *base_addr;
static u64                      rsrc_start;
static u64                      rsrc_len;


static unsigned char xidata[]=
{
		0x96,0X01,
		0x19,0X87,
		0xFF,20, //10ms
		0x26,0X80,
		0x1B,0X0c,
		0x43,0X00,
		0x20,0X00,
		0x1f,0X07,
		0x44,0x7F,
		0x45,0x14,
		0x1d,0X05,
		0x1e,0X00,
		0x1c,0X04,
		0x1b,0X14,
		0xFF, 80, //40ms
		0x43,0X80,
		0x42,0X08,
		0x23,0X95,
		0x24,0X95,
		0x25,0Xff,
		0x21,0X10,
		
		0x2b,0X00,
		0x95,0X01,
		
		0x1a,0X00,
		0x93,0X0f,
		0x70,0X66,
		0x18,0X01,
	    0x46,0X86,
		0x47,0X60,
		0x48,0X01,
		0x49,0X67,
		0x4a,0X46,
		0x4b,0x13,
		0x4c,0X01,
		0x4d,0X67,
		0x4e,0X00,
		0x4f,0X13,
		0x50,0X02,
		0x51,0X00,
		0x38,0X00, //RGB interface contral    
		0x39,0X00,
		0x27,0X02,
		0x28,0x03,
		0x29,0x08,
		0x2a,0x08,
		0x2c,0x08,
		0x2d,0x08,
		0x35,0x09,
		0x36,0x09,
		0x91,0x14,
		0x37,0x00,
		0x01,0x06,
		0x3a,0xa1,
		0x3b,0xa1,
		0x3c,0xa0,
		0x3d,0x00,
		0x3e,0x2d,
		0x40,0x03,
		0x41,0xcc,
		0x0a,0x00,
		0x0b,0x00,
		0x0c,0x01,
		0x0d,0x3f,
		0x0e,0x00,
		0x0f,0x00,
		0x10,0x01,
		0x11,0x40,
		0x12,0x00,
		0x13,0x00,
		0x14,0x00,
		0x15,0x00,
		0x02,0x00,
		0x03,0x00,
		0x04,0x00,
		0x05,0xef,
		0x06,0x00,
		0x07,0x00,
		0x08,0x01,
		0x09,0x3f,
		0x16,0x08,
		0x72,0x00,
		0x22,0X00,  // GRAM Write Data
		0xFF,120, //60ms
		0x94,0x0a,
		0x90,0x7f,
		0x26,0x84,
		0xFF, 80, //40ms
		0x26,0xa4,
		0x26,0xac,
		0xFF, 80, //40ms
		0x26,0xbc,
		0x96,0x00
};

void CS0_CON(char en)
{
		u32 tmp;
		if(en== 0){
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp &=~SYS_CS0_CON, base_addr + SIFCCON0);
		} else {
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp |=SYS_CS0_CON, base_addr + SIFCCON0);
		}
}
void OE_CON(char en)
{
		u32 tmp;
		if(en== 0){
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp &=~SYS_OE_CON, base_addr + SIFCCON0);
		} else {
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp |=SYS_OE_CON, base_addr + SIFCCON0);
		}
}
void WR_CON(char en)
{
		u32 tmp;
		if(en== 0){
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp &=~SYS_WR_CON, base_addr + SIFCCON0);
		} else {
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp |=SYS_WR_CON, base_addr + SIFCCON0);
		}
}
void RS_CON(char en)
{
		u32 tmp;
		if(en== 0){
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp &=~SYS_RS_CON, base_addr + SIFCCON0);
		} else {
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp |=SYS_RS_CON, base_addr + SIFCCON0);
		}
}
void SCOMEN(char en)
{
		u32 tmp;
		if(en== 0){
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp &=~SYS_SCOMEN, base_addr + SIFCCON0);
		} else {
				tmp=readl(base_addr + SIFCCON0);
				writel(tmp |=SYS_SCOMEN, base_addr + SIFCCON0);
		}
}



static int hx8347_write_reg(u8 reg, u8 data)
{
	SCOMEN(ENABLE);
	
	RS_CON(INDEX);
	CS0_CON(ENABLE);
	
	writel(reg, base_addr + SIFCCON1);
	WR_CON(ENABLE);
	WR_CON(DISABLE);
	
	RS_CON(DATA);
	
	writel(data, base_addr + SIFCCON1);
	WR_CON(ENABLE);
	WR_CON(DISABLE);
	
	CS0_CON(DISABLE);
	SCOMEN(DISABLE);
	return 0;
}

#if 0
static int hx8352_read_reg(u8 reg)
{
		unsigned char val;
		
		SCOMEN(ENABLE);
		
		RS_CON(INDEX);
		CS0_CON(ENABLE);
		
		SIFCCON1_REG=reg;
		WR_CON(ENABLE);
		WR_CON(DISABLE);
		
		RS_CON(DATA);
		OE_CON(ENABLE);
		val =(unsigned char)(SIFCCON2_REG);
		OE_CON(DISABLE);
		
		CS0_CON(DISABLE);
		SCOMEN(DISABLE);
		
		return val;

}
#endif
void set_write_gram_mode(void)
{
		SCOMEN(ENABLE);
		
		RS_CON(INDEX);
		CS0_CON(ENABLE);
	 
		writel(0x22, base_addr + SIFCCON1);
		WR_CON(ENABLE);
		WR_CON(DISABLE);
		
		RS_CON(DATA);
		
		CS0_CON(DISABLE);
		SCOMEN(DISABLE);
}



static void l10tlm012_reg_init(void)
{
	int i;
	for (i = 0; i < ARRAY_SIZE(xidata);i += 2) {
			if (xidata[i] == 0xff)
					msleep(xidata[i+1]);
			else if(xidata[i] == 0x22)
					set_write_gram_mode();
			else hx8347_write_reg(xidata[i], xidata[i+1]);
	}
}


static int l10tlm012_power_on(struct l10tlm012 *lcd)
{
	/* FIXME : reset LCD */
	s3c2410_gpio_cfgpin(S3C2410_GPE1, S3C2410_GPIO_OUTPUT);
    s3c2410_gpio_setpin(S3C2410_GPE1, 1);
    s3c2410_gpio_pullup(S3C2410_GPE1, S3C24XX_GPIO_PULL_UP);
	msleep(1);
    s3c2410_gpio_setpin(S3C2410_GPB1, 0);
	msleep(1);
    s3c2410_gpio_setpin(S3C2410_GPB1, 1);
	msleep(120);

	/* power on sequence */
	l10tlm012_reg_init();
	//l10tlm012_set_gamma(lcd);
	//l10tlm012_late_init(lcd);
	return 0;
}

static int l10tlm012_power_off(struct l10tlm012 *lcd)
{
	/* FXIME */
	return 0;
}

static int l10tlm012_power(struct l10tlm012 *lcd, int power)
{
        int ret = 0;

        if (POWER_IS_ON(power) && !POWER_IS_ON(lcd->power))
                ret = l10tlm012_power_on(lcd);
        else if (!POWER_IS_ON(power) && POWER_IS_ON(lcd->power))
                ret = l10tlm012_power_off(lcd);

        if (!ret)
                lcd->power = power;

        return ret;
}

static int l10tlm012_set_power(struct lcd_device *ld, int power)
{
	struct l10tlm012 *lcd = lcd_get_data(ld);
	return l10tlm012_power(lcd, power);
}

static int l10tlm012_get_power(struct lcd_device *ld)
{
	struct l10tlm012 *lcd = lcd_get_data(ld);
	return lcd->power;
}

static struct lcd_ops ltv_ops = {
	.get_power	= l10tlm012_get_power,
	.set_power	= l10tlm012_set_power,
};

static int __devinit l10tlm012_probe(struct platform_driver *pdev)
{
	struct l10tlm012  *lcd;
	struct lcd_device *ld;
	int ret;
	
	rsrc_start=0x4c800000;
	rsrc_len=0x4000;

	lcd = kzalloc(sizeof(struct l10tlm012), GFP_KERNEL);
	if (!lcd)
		return -ENOMEM;

		if (!request_mem_region(rsrc_start, rsrc_len,"l10tlm012")){
			return -EBUSY;
	}
	
	base_addr = ioremap(rsrc_start,rsrc_len);
	if (!base_addr) {
			ret = -ENOMEM;
			goto err_remap;
	}

	lcd->power = FB_BLANK_POWERDOWN;

	ld = lcd_device_register("l10tlm012",&pdev->dev , lcd, &ltv_ops);
	if (IS_ERR(ld)) {
		ret = PTR_ERR(ld);
		goto out_free_lcd;
	}
	lcd->ld = ld;

	ret = l10tlm012_power(lcd, FB_BLANK_UNBLANK);
	if (ret)
		goto out_unregister;
	return 0;
err_mem:
	release_mem_region(rsrc_start, rsrc_len);
	kfree(base_addr);
out_unregister:
	lcd_device_unregister(ld);
out_free_lcd:
	kfree(lcd);
	return ret;
}

static int __devexit l10tlm012_remove(struct spi_device *spi)
{
	l10tlm012_power(lcd, FB_BLANK_POWERDOWN);
	lcd_device_unregister(lcd->ld);
	kfree(lcd);

	return 0;
}

#ifdef CONFIG_PM
static int l10tlm012_suspend(struct spi_device *spi, pm_message_t state)
{
	struct l10tlm012 *lcd = dev_get_drvdata(&spi->dev);

	return l10tlm012_power(lcd, FB_BLANK_POWERDOWN);
}

static int l10tlm012_resume(struct spi_device *spi)
{
	struct l10tlm012 *lcd = dev_get_drvdata(&spi->dev);

	return l10tlm012_power(lcd, FB_BLANK_UNBLANK);
}
#else
#define l10tlm012_suspend	NULL
#define l10tlm012_resume		NULL
#endif

/* Power down all displays on reboot, poweroff or halt */
static void l10tlm012_shutdown(struct spi_device *spi)
{
	struct l10tlm012 *lcd = dev_get_drvdata(&spi->dev);

	l10tlm012_power(lcd, FB_BLANK_POWERDOWN);
}

static struct platform_driver l10tlm012_driver = {
	.driver = {
		.name		= "l10tlm012",
		.owner		= THIS_MODULE,
	},

	.probe		= l10tlm012_probe,
	.remove		= __devexit_p(l10tlm012_remove),
	.shutdown	= l10tlm012_shutdown,
	.suspend	= l10tlm012_suspend,
	.resume		= l10tlm012_resume,
};

static int __init l10tlm012_init(void)
{
	return platform_driver(&l10tlm012_driver);
}

static void __exit l10tlm012_exit(void)
{
	platform_driver_unregister(&l10tlm012_driver);
}
module_init(l10tlm012_init);
module_exit(l10tlm012_exit);

MODULE_LICENSE("GPL");
