/*
 * Power control for Himax 
 *
 * Copyright (C) 2006, 2007 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/delay.h>
#include <linux/err.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/lcd.h>
#include <linux/module.h>
#include <linux/spi/spi.h>
#include <asm/mach-types.h>

/* FIXME */
#include <mach/regs-gpio.h>

#define SYSTEM_REV_YUP150N      5 

#define SYSTEM_REV_YUP120SN     1 
#define SYSTEM_REV_YUP120N      2 
#define SYSTEM_REV_YUP160W      3 

#define POWER_IS_ON(pwr)	((pwr) <= FB_BLANK_NORMAL)

struct xt32013a {
	struct spi_device	*spi;
	int			power;
	struct lcd_device	*ld;

	u8			device_id; /* 011101 */
};

static int hx8352_write_reg(struct xt32013a *lcd, u8 reg, u8 data)
{
	ssize_t retval;
	u8	cmd[2];

	cmd[0] = lcd->device_id << 2;
	cmd[1] = reg;
	retval = spi_write(lcd->spi, cmd, 2);
	if (retval < 0) {
		dev_err(&lcd->spi->dev, "error %d writing reg index%02X\n",retval, reg);
		return retval;
	}

	cmd[0] = (lcd->device_id << 2) | 0x2;
	cmd[1] = data;
	retval = spi_write(lcd->spi, cmd, 2);
	if (retval  < 0) {
		dev_err(&lcd->spi->dev, "error %d writing data at index%02X\n",retval, reg);
		return retval;
	}

	return 0;
}

#if 0
static int hx8352_read_reg(struct xt32013a *lcd, u8 reg, u8 *data)
{
	ssize_t retval;
	u8	cmd[3];

	cmd[0] = lcd->device_id << 2;
	cmd[1] = reg;
	retval = spi_write(lcd->spi, cmd, 2);
	if (retval < 0) {
		dev_err(&lcd->spi->dev, "error %d reading reg index%02X\n",retval, reg);
		return retval;
	}

	cmd[0] = (lcd->device_id << 2) | 0x3;
	retval = spi_write_then_read(lcd->spi, cmd, 1, cmd+1, 2);
	if (retval  < 0) {
		dev_err(&lcd->spi->dev, "error %d reading data at index%02X\n",retval, reg);
		return retval;
	}

	*data = cmd[2];
	return 0;
}
#endif

static void xt32013a_early_init(struct xt32013a *lcd)
{
	static u8 reg_vals[][2] = {
		{0x17,0X91},	/* OSC Control 1 Register */
		{0x83,0X02},
		{0x85,0X03},
		{0x8B,0X01},
		{0x8C,0X93},
		{0x91,0X01},
		{0x83,0X00},
	};
	int i;

	for (i = 0; i < ARRAY_SIZE(reg_vals); i++)
		hx8352_write_reg(lcd, reg_vals[i][0], reg_vals[i][1]);
}
static void xt32013a_default_set_gamma(struct xt32013a *lcd)
{
	static u8 gamma_regs[][2] = {

		{0x3E,0xC4},
		{0x3F,0x44},
		{0x40,0X22},
		{0x41,0X57},
		{0x42,0X03},
		{0x43,0X47},
		{0x44,0X02},
		{0x45,0X55},
		{0x46,0X07},
		{0x47,0X48},
		{0x48,0X07},
		{0x49,0X88},
		{0x17,0X91},	/* OSC Control 1 Register */
		{0x2B,0XF9},
	};
	int i;

	for (i = 0; i < ARRAY_SIZE(gamma_regs); i++) {
		hx8352_write_reg(lcd, gamma_regs[i][0], gamma_regs[i][1]);
	}
	msleep(10);
}
static void xt32013a_set_gamma(struct xt32013a *lcd)
{
	static u8 gamma_regs[][2] = {
		{0x3E,0xD0},
		{0x3F,0x17},
		{0x40,0X00},
		{0x41,0X43},
		{0x42,0X16},
		{0x43,0X16},
		{0x44,0X43},
		{0x45,0X77},
		{0x46,0X00},
		{0x47,0X1E},
		{0x48,0X0F},
		{0x49,0X00},
		
		{0x17,0X91},	/* OSC Control 1 Register */
		{0x2B,0XF9},
	};
	int i;

	for (i = 0; i < ARRAY_SIZE(gamma_regs); i++) {
		hx8352_write_reg(lcd, gamma_regs[i][0], gamma_regs[i][1]);
	}
	msleep(10);
}

static void xt32013a_late_init(struct xt32013a *lcd)
{
	static u8 regs_step1[][2] = {

		/*USER SETTING*/
		{0x1B,0X13},
		{0x1A,0X11},
		{0x1C,0X0A},	/* VCOMH=VREG1*1.094=5.4  */
		{0x1F,0X60},
		{0x19,0X0A},
		{0x19,0X1A},
		{0x19,0X12},
		{0x1E,0X36},
		{0x3C,0XC0},	/* Digital gamma correction enable */
		/*DISPLAY ON*/
		{0x3C,0XC0},
		{0x3D,0XC0},
		{0x34,0X38},
		{0x35,0X38},
		{0x24,0XA8},
		{0x24,0XB8},
	};
	static u8 regs_step2[][2] = {
		{0x24,0X3C},	/* Display Control 2 Register */
		{0x16,0X08},	/* MY:0 MX:0 GS:0 BGR:1 */
		{0x01,0X02},
		{0x55,0X00},
	};
	int i;

	for (i = 0; i < ARRAY_SIZE(regs_step1); i++){
		hx8352_write_reg(lcd, regs_step1[i][0], regs_step1[i][1]);
		if(i== 3)msleep(20);
		if(i== 5)msleep(40);
		if(i== 6)msleep(40);
		if(i== 7)msleep(100);
		if(i== 13)msleep(40);

	}
	msleep(40);
	for (i = 0; i < ARRAY_SIZE(regs_step2); i++)
		hx8352_write_reg(lcd, regs_step2[i][0], regs_step2[i][1]);
}

static int xt32013a_power_on(struct xt32013a *lcd)
{
    	if(system_rev == SYSTEM_REV_YUP120N || system_rev == SYSTEM_REV_YUP160W){
	    	/* FIXME : reset LCD */
	    	s3c2410_gpio_cfgpin(S3C2410_GPF3, S3C2410_GPIO_OUTPUT);
		s3c2410_gpio_setpin(S3C2410_GPF3, 0);
		s3c2410_gpio_pullup(S3C2410_GPF3, S3C24XX_GPIO_PULL_UP);
		msleep(10);
		s3c2410_gpio_setpin(S3C2410_GPF3, 1);
	} else {
	    	/* FIXME : reset LCD */
	    	s3c2410_gpio_cfgpin(S3C2410_GPB1, S3C2410_GPIO_OUTPUT);        
		s3c2410_gpio_setpin(S3C2410_GPB1, 0);
		s3c2410_gpio_pullup(S3C2410_GPB1, S3C24XX_GPIO_PULL_UP);
		msleep(10);
		s3c2410_gpio_setpin(S3C2410_GPB1, 1);
	}
	
	if ((machine_is_yup150()) && (system_rev == SYSTEM_REV_YUP150N))
	    	msleep(10);

	/* power on sequence */
	xt32013a_early_init(lcd);
	//if(system_rev== SYSTEM_REV_YUP110N)
	//	xt32013a_default_set_gamma(lcd);
	//else
		xt32013a_set_gamma(lcd);
	xt32013a_late_init(lcd);
	return 0;
}

static int xt32013a_power_off(struct xt32013a *lcd)
{
	/* FXIME */
	return 0;
}

static int xt32013a_power(struct xt32013a *lcd, int power)
{
        int ret = 0;

        if (POWER_IS_ON(power) && !POWER_IS_ON(lcd->power))
                ret = xt32013a_power_on(lcd);
        else if (!POWER_IS_ON(power) && POWER_IS_ON(lcd->power))
                ret = xt32013a_power_off(lcd);

        if (!ret)
                lcd->power = power;

        return ret;
}

static int xt32013a_set_power(struct lcd_device *ld, int power)
{
	struct xt32013a *lcd = lcd_get_data(ld);
	return xt32013a_power(lcd, power);
}

static int xt32013a_get_power(struct lcd_device *ld)
{
	struct xt32013a *lcd = lcd_get_data(ld);
	return lcd->power;
}

static struct lcd_ops ltv_ops = {
	.get_power	= xt32013a_get_power,
	.set_power	= xt32013a_set_power,
};

static int __devinit xt32013a_probe(struct spi_device *spi)
{
	struct xt32013a *lcd;
	struct lcd_device *ld;
	int ret;

	lcd = kzalloc(sizeof(struct xt32013a), GFP_KERNEL);
	if (!lcd)
		return -ENOMEM;

	lcd->spi = spi;
	lcd->power = FB_BLANK_POWERDOWN;
	lcd->device_id = 0x1D; /* FIXME : 011101 */

	ld = lcd_device_register("xt32013a", &spi->dev, lcd, &ltv_ops);
	if (IS_ERR(ld)) {
		ret = PTR_ERR(ld);
		goto out_free_lcd;
	}
	lcd->ld = ld;

	ret = xt32013a_power(lcd, FB_BLANK_UNBLANK);
	if (ret)
		goto out_unregister;

	dev_set_drvdata(&spi->dev, lcd);

	return 0;

out_unregister:
	lcd_device_unregister(ld);
out_free_lcd:
	kfree(lcd);
	return ret;
}

static int __devexit xt32013a_remove(struct spi_device *spi)
{
	struct xt32013a *lcd = dev_get_drvdata(&spi->dev);

	xt32013a_power(lcd, FB_BLANK_POWERDOWN);
	lcd_device_unregister(lcd->ld);
	kfree(lcd);

	return 0;
}

#ifdef CONFIG_PM
static int xt32013a_suspend(struct spi_device *spi, pm_message_t state)
{
	struct xt32013a *lcd = dev_get_drvdata(&spi->dev);

	return xt32013a_power(lcd, FB_BLANK_POWERDOWN);
}

static int xt32013a_resume(struct spi_device *spi)
{
	struct xt32013a *lcd = dev_get_drvdata(&spi->dev);

	return xt32013a_power(lcd, FB_BLANK_UNBLANK);
}
#else
#define xt32013a_suspend	NULL
#define xt32013a_resume		NULL
#endif

/* Power down all displays on reboot, poweroff or halt */
static void xt32013a_shutdown(struct spi_device *spi)
{
	struct xt32013a *lcd = dev_get_drvdata(&spi->dev);

	xt32013a_power(lcd, FB_BLANK_POWERDOWN);
}

static struct spi_driver xt32013a_driver = {
	.driver = {
		.name		= "xt32013a",
		.bus		= &spi_bus_type,
		.owner		= THIS_MODULE,
	},

	.probe		= xt32013a_probe,
	.remove		= __devexit_p(xt32013a_remove),
	.shutdown	= xt32013a_shutdown,
	.suspend	= xt32013a_suspend,
	.resume		= xt32013a_resume,
};

static int __init xt32013a_init(void)
{
	return spi_register_driver(&xt32013a_driver);
}

static void __exit xt32013a_exit(void)
{
	spi_unregister_driver(&xt32013a_driver);
}
module_init(xt32013a_init);
module_exit(xt32013a_exit);

MODULE_LICENSE("GPL");
