/*
 * Copyright (c) 2008-2009, Kionix, Inc. All Rights Reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __DMARD03_H__
#define __DMARD03_H__

#include <linux/ioctl.h>  /* For IOCTL macros */

/** This define controls compilation of the master device interface */

/* CONTROL REGISTER 1 BITS */
/* Moving average filter of various lengths */
#define N_AO8			0x00	/* 8 */
#define N_AO4			0x01	/* 4 */
#define N_AO2			0x02	/* 2 */
#define N_AO1			0x03	/* 1 */
/* internal ADC sampling period */
#define TMS_342			0x00	/* 342Hz/2.925ms */
#define TMS_684			0x04	/* 684Hz/1.462ms */
/* High-G Latch Control */
#define HGL			0x08	/* high-G detection OFF */
/* INT1 and INT2 Source Configuer */
#define INT1_GND		0x00
#define INT1_FREEFALL		0x10
#define INT1_CLICK		0x20
#define INT1_HIGH_G		0x30
#define INT2_GND		0x00
#define INT2_FREEFALL		0x40
#define INT2_CLICK		0x80
#define INT2_HIGH_G		0xC0

/* CONTROL REGISTER 2 BITS */
/* click interrupt enable */
#define CLZEN			0x01	
#define CLYEN			0x02	
#define CLXEN			0x04	
/* High-G Detection enable */
#define HGZEN			0x10
#define HGYEN			0x20
#define HGXEN			0x40
/* multiple axial detections */
#define HGAO_OR_LOGIC		0x00 /*HGAO setting  before int1 and int2 setting */
#define HGAO_AND_LOGIC		0x80

/* CONTROL REGISTER 3 BITS */
/* FFth :freefall threshold/ effect setting range is from 0xf(937.5mg) to 0x1(62.5mg) */
/* HGth :High-G threshold/ effect setting range is from 0xC(3G) to 0x1(250mg) */

/* CONTROL REGISTER 4 BITS */
/* Clickth :Click threshold/ effect setting range is from 0xC(3G) to 0x1(250mg) */

/* INTERRUPT CONTROL REGISTER 1 BITS */
#define DMARD03_IEN		0x10	/* interrupt enable */
#define DMARD03_IEA		0x08	/* interrupt polarity */
#define DMARD03_IEL		0x04	/* interrupt response */


#ifdef __KERNEL__
struct dmard03_platform_data {

	u8 g_range;

	u8 ctrl_reg1_init;
	u8 ctrl_reg2_init;
	u8 ctrl_reg3_init;
	u8 ctrl_reg4_init;
	
	int (*init)(void);
	void (*exit)(void);
	int (*power_on)(void);
	int (*power_off)(void);

	int irq;
};
#endif /* __KERNEL__ */

#endif  /* __KXTE9_H__ */

